@@ -1,4 +1,4 @@
-/* $Id: chars.c,v 1.36 2005/03/26 06:54:36 dolorous Exp $ */
+/* $Id: chars.c,v 1.75 2005/06/29 18:17:54 dolorous Exp $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -28,7 +29,6 @@
 #include <ctype.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 #ifdef NANO_WIDE
 #ifdef HAVE_WCHAR_H
@@ -39,6 +39,22 @@
 #endif
 #endif
 
+#ifndef HAVE_ISBLANK
+/* This function is equivalent to isblank(). */
+bool nisblank(int c)
+{
+    return isspace(c) && (c == '\t' || !is_cntrl_char(c));
+}
+#endif
+
+#if !defined(HAVE_ISWBLANK) && defined(NANO_WIDE)
+/* This function is equivalent to iswblank(). */
+bool niswblank(wchar_t wc)
+{
+    return iswspace(wc) && (wc == '\t' || !is_cntrl_wchar(wc));
+}
+#endif
+
 /* Return TRUE if the value of c is in byte range, and FALSE
  * otherwise. */
 bool is_byte(int c)
@@ -46,19 +62,13 @@ bool is_byte(int c)
     return ((unsigned int)c == (unsigned char)c);
 }
 
-/* This function is equivalent to isalnum(). */
-bool is_alnum_char(int c)
-{
-    return isalnum(c);
-}
-
 /* This function is equivalent to isalnum() for multibyte characters. */
 bool is_alnum_mbchar(const char *c)
 {
     assert(c != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
@@ -67,30 +77,10 @@ bool is_alnum_mbchar(const char *c)
 	    wc = (unsigned char)*c;
 	}
 
-	return is_alnum_wchar(wc);
+	return iswalnum(wc);
     } else
 #endif
-	return is_alnum_char((unsigned char)*c);
-}
-
-#ifdef NANO_WIDE
-/* This function is equivalent to isalnum() for wide characters. */
-bool is_alnum_wchar(wchar_t wc)
-{
-    return iswalnum(wc);
-}
-#endif
-
-/* This function is equivalent to isblank(). */
-bool is_blank_char(int c)
-{
-    return
-#ifdef HAVE_ISBLANK
-	isblank(c)
-#else
-	isspace(c) && (c == '\t' || !is_cntrl_char(c))
-#endif
-	;
+	return isalnum((unsigned char)*c);
 }
 
 /* This function is equivalent to isblank() for multibyte characters. */
@@ -99,7 +89,7 @@ bool is_blank_mbchar(const char *c)
     assert(c != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
@@ -108,26 +98,12 @@ bool is_blank_mbchar(const char *c)
 	    wc = (unsigned char)*c;
 	}
 
-	return is_blank_wchar(wc);
+	return iswblank(wc);
     } else
 #endif
-	return is_blank_char((unsigned char)*c);
+	return isblank((unsigned char)*c);
 }
 
-#ifdef NANO_WIDE
-/* This function is equivalent to isblank() for wide characters. */
-bool is_blank_wchar(wchar_t wc)
-{
-    return
-#ifdef HAVE_ISWBLANK
-	iswblank(wc)
-#else
-	iswspace(wc) && (wc == '\t' || !is_cntrl_wchar(wc))
-#endif
-	;
-}
-#endif
-
 /* This function is equivalent to iscntrl(), except in that it also
  * handles control characters with their high bits set. */
 bool is_cntrl_char(int c)
@@ -136,6 +112,16 @@ bool is_cntrl_char(int c)
 	(127 <= c && c < 160);
 }
 
+#ifdef NANO_WIDE
+/* This function is equivalent to iscntrl() for wide characters, except
+ * in that it also handles wide control characters with their high bits
+ * set. */
+bool is_cntrl_wchar(wchar_t wc)
+{
+    return (0 <= wc && wc < 32) || (127 <= wc && wc < 160);
+}
+#endif
+
 /* This function is equivalent to iscntrl() for multibyte characters,
  * except in that it also handles multibyte control characters with
  * their high bits set. */
@@ -144,7 +130,7 @@ bool is_cntrl_mbchar(const char *c)
     assert(c != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
@@ -159,19 +145,41 @@ bool is_cntrl_mbchar(const char *c)
 	return is_cntrl_char((unsigned char)*c);
 }
 
-#ifdef NANO_WIDE
-/* This function is equivalent to iscntrl() for wide characters, except
- * in that it also handles wide control characters with their high bits
- * set. */
-bool is_cntrl_wchar(wchar_t wc)
+/* This function is equivalent to ispunct() for multibyte characters. */
+bool is_punct_mbchar(const char *c)
 {
-    return (0 <= wc && wc < 32) || (127 <= wc && wc < 160);
-}
-#endif
+    assert(c != NULL);
 
-/* c is a control character.  It displays as ^@, ^?, or ^[ch] where ch
+#ifdef NANO_WIDE
+    if (ISSET(USE_UTF8)) {
+	wchar_t wc;
+	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
+
+	if (c_mb_len <= 0) {
+	    mbtowc(NULL, NULL, 0);
+	    wc = (unsigned char)*c;
+	}
+
+	return iswpunct(wc);
+    } else
+#endif
+	return ispunct((unsigned char)*c);
+}
+
+/* Return TRUE for a multibyte character found in a word (currently only
+ * an alphanumeric or punctuation character, and only the latter if
+ * allow_punct is TRUE) and FALSE otherwise. */
+bool is_word_mbchar(const char *c, bool allow_punct)
+{
+    assert(c != NULL);
+
+    return is_alnum_mbchar(c) || (allow_punct ? is_punct_mbchar(c) :
+	FALSE);
+}
+
+/* c is a control character.  It displays as ^@, ^?, or ^[ch], where ch
  * is c + 64.  We return that character. */
-unsigned char control_rep(unsigned char c)
+char control_rep(char c)
 {
     /* Treat newlines embedded in a line as encoded nulls. */
     if (c == '\n')
@@ -182,48 +190,9 @@ unsigned char control_rep(unsigned char c)
 	return c + 64;
 }
 
-/* c is a multibyte control character.  It displays as ^@, ^?, or ^[ch]
- * where ch is c + 64.  We return that multibyte character. */
-char *control_mbrep(const char *c, char *crep, int *crep_len)
-{
-    assert(c != NULL);
-
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
-	wchar_t wc, wcrep;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX), crep_mb_len;
-
-	if (c_mb_len <= 0) {
-	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
-	}
-
-	wcrep = control_wrep(wc);
-
-	crep_mb_len = wctomb(crep, wcrep);
-
-	if (crep_mb_len <= 0) {
-	    wctomb(NULL, 0);
-	    crep_mb_len = 0;
-	}
-
-	*crep_len = crep_mb_len;
-
-	return crep;
-    } else {
-#endif
-	*crep_len = 1;
-	*crep = control_rep((unsigned char)*c);
-
-	return crep;
-#ifdef NANO_WIDE
-    }
-#endif
-}
-
-#ifdef NANO_WIDE
-/* c is a wide control character.  It displays as ^@, ^?, or ^[ch] where
- * ch is c + 64.  We return that wide character. */
+/* c is a wide control character.  It displays as ^@, ^?, or ^[ch],
+ * where ch is c + 64.  We return that wide character. */
 wchar_t control_wrep(wchar_t wc)
 {
     /* Treat newlines embedded in a line as encoded nulls. */
@@ -236,13 +205,45 @@ wchar_t control_wrep(wchar_t wc)
 }
 #endif
 
+/* c is a multibyte control character.  It displays as ^@, ^?, or ^[ch],
+ * where ch is c + 64.  We return that multibyte character. */
+char *control_mbrep(const char *c, char *crep, int *crep_len)
+{
+    assert(c != NULL && crep != NULL && crep_len != NULL);
+
+#ifdef NANO_WIDE
+    if (ISSET(USE_UTF8)) {
+	wchar_t wc;
+
+	if (mbtowc(&wc, c, MB_CUR_MAX) <= 0) {
+	    mbtowc(NULL, NULL, 0);
+	    wc = (unsigned char)*c;
+	}
+
+	*crep_len = wctomb(crep, control_wrep(wc));
+
+	if (*crep_len <= 0) {
+	    wctomb(NULL, 0);
+	    *crep_len = 0;
+	}
+    } else {
+#endif
+	*crep_len = 1;
+	*crep = control_rep(*c);
+#ifdef NANO_WIDE
+    }
+#endif
+
+    return crep;
+}
+
 /* This function is equivalent to wcwidth() for multibyte characters. */
 int mbwidth(const char *c)
 {
     assert(c != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX), width;
 
@@ -252,6 +253,7 @@ int mbwidth(const char *c)
 	}
 
 	width = wcwidth(wc);
+
 	if (width == -1)
 	    width++;
 
@@ -266,14 +268,16 @@ int mb_cur_max(void)
 {
     return
 #ifdef NANO_WIDE
-	!ISSET(NO_UTF8) ? MB_CUR_MAX :
+	ISSET(USE_UTF8) ? MB_CUR_MAX :
 #endif
 	1;
 }
 
 /* Convert the value in chr to a multibyte character with the same
- * wide character value as chr.  Return the (dynamically allocated)
- * multibyte character and its length. */
+ * wide character value as chr, if possible.  If the conversion
+ * succeeds, return the (dynamically allocated) multibyte character and
+ * its length.  Otherwise, return an undefined (dynamically allocated)
+ * multibyte character and a length of zero. */
 char *make_mbchar(int chr, int *chr_mb_len)
 {
     char *chr_mb;
@@ -281,7 +285,7 @@ char *make_mbchar(int chr, int *chr_mb_len)
     assert(chr_mb_len != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	chr_mb = charalloc(MB_CUR_MAX);
 	*chr_mb_len = wctomb(chr_mb, chr);
 
@@ -292,8 +296,7 @@ char *make_mbchar(int chr, int *chr_mb_len)
     } else {
 #endif
 	*chr_mb_len = 1;
-	chr_mb = charalloc(1);	
-	*chr_mb = (char)chr;
+	chr_mb = mallocstrncpy(NULL, (char *)&chr, 1);
 #ifdef NANO_WIDE
     }
 #endif
@@ -301,58 +304,6 @@ char *make_mbchar(int chr, int *chr_mb_len)
     return chr_mb;
 }
 
-#if defined(ENABLE_NANORC) || defined(ENABLE_EXTRA)
-/* Convert the string str to a valid multibyte string with the same wide
- * character values as str.  Return the (dynamically allocated)
- * multibyte string. */
-char *make_mbstring(const char *str)
-{
-    assert(str != NULL);
-
-#ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
-	char *chr_mb = charalloc(MB_CUR_MAX);
-	int chr_mb_len;
-	char *str_mb = charalloc((MB_CUR_MAX * strlen(str)) + 1);
-	size_t str_mb_len = 0;
-
-	while (*str != '\0') {
-	    bool bad_char;
-	    int i;
-
-	    chr_mb_len = parse_mbchar(str, chr_mb, &bad_char, NULL);
-
-	    if (bad_char) {
-		char *bad_chr_mb;
-		int bad_chr_mb_len;
-
-		bad_chr_mb = make_mbchar((unsigned char)*chr_mb,
-		    &bad_chr_mb_len);
-
-		for (i = 0; i < bad_chr_mb_len; i++)
-		    str_mb[str_mb_len + i] = bad_chr_mb[i];
-		str_mb_len += bad_chr_mb_len;
-
-		free(bad_chr_mb);
-	    } else {
-		for (i = 0; i < chr_mb_len; i++)
-		    str_mb[str_mb_len + i] = chr_mb[i];
-		str_mb_len += chr_mb_len;
-	    }
-
-	    str += chr_mb_len;
-	}
-
-	free(chr_mb);
-	null_at(&str_mb, str_mb_len);
-
-	return str_mb;
-     } else
-#endif
-	return mallocstrcpy(NULL, str);
-}
-#endif
-
 /* Parse a multibyte character from buf.  Return the number of bytes
  * used.  If chr isn't NULL, store the multibyte character in it.  If
  * bad_chr isn't NULL, set it to TRUE if we have a bad multibyte
@@ -369,7 +320,7 @@ int parse_mbchar(const char *buf, char *chr, bool *bad_chr, size_t
 	*bad_chr = FALSE;
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	/* Get the number of bytes in the multibyte character. */
 	buf_mb_len = mblen(buf, MB_CUR_MAX);
 
@@ -505,7 +456,7 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
     }
 
     if (n > 0)
-	return (tolower(*s1) - tolower(*s2));
+	return tolower(*s1) - tolower(*s2);
     else
 	return 0;
 }
@@ -516,7 +467,7 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
 int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 {
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	char *s1_mb = charalloc(MB_CUR_MAX);
 	char *s2_mb = charalloc(MB_CUR_MAX);
 	wchar_t ws1, ws2;
@@ -524,6 +475,7 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	assert(s1 != NULL && s2 != NULL);
 
 	while (n > 0 && *s1 != '\0' && *s2 != '\0') {
+	    bool bad_s1_mb = FALSE, bad_s2_mb = FALSE;
 	    int s1_mb_len, s2_mb_len;
 
 	    s1_mb_len = parse_mbchar(s1, s1_mb, NULL, NULL);
@@ -531,6 +483,7 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	    if (mbtowc(&ws1, s1_mb, s1_mb_len) <= 0) {
 		mbtowc(NULL, NULL, 0);
 		ws1 = (unsigned char)*s1_mb;
+		bad_s1_mb = TRUE;
 	    }
 
 	    s2_mb_len = parse_mbchar(s2, s2_mb, NULL, NULL);
@@ -538,9 +491,11 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	    if (mbtowc(&ws2, s2_mb, s2_mb_len) <= 0) {
 		mbtowc(NULL, NULL, 0);
 		ws2 = (unsigned char)*s2_mb;
+		bad_s2_mb = TRUE;
 	    }
 
-	    if (n == 0 || towlower(ws1) != towlower(ws2))
+	    if (n == 0 || bad_s1_mb != bad_s2_mb ||
+		towlower(ws1) != towlower(ws2))
 		break;
 
 	    s1 += s1_mb_len;
@@ -551,7 +506,7 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	free(s1_mb);
 	free(s2_mb);
 
-	return (towlower(ws1) - towlower(ws2));
+	return towlower(ws1) - towlower(ws2);
     } else
 #endif
 	return strncasecmp(s1, s2, n);
@@ -582,7 +537,7 @@ const char *nstrcasestr(const char *haystack, const char *needle)
 const char *mbstrcasestr(const char *haystack, const char *needle)
 {
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	char *r_mb = charalloc(MB_CUR_MAX);
 	char *q_mb = charalloc(MB_CUR_MAX);
 	wchar_t wr, wq;
@@ -595,11 +550,14 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 	    int r_mb_len, q_mb_len;
 
 	    while (*q != '\0') {
+		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
+
 		r_mb_len = parse_mbchar(r, r_mb, NULL, NULL);
 
 		if (mbtowc(&wr, r_mb, r_mb_len) <= 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wr = (unsigned char)*r;
+		    bad_r_mb = TRUE;
 		}
 
 		q_mb_len = parse_mbchar(q, q_mb, NULL, NULL);
@@ -607,9 +565,11 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 		if (mbtowc(&wq, q_mb, q_mb_len) <= 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wq = (unsigned char)*q;
+		    bad_q_mb = TRUE;
 		}
 
-		if (towlower(wr) != towlower(wq))
+		if (bad_r_mb != bad_q_mb ||
+			towlower(wr) != towlower(wq))
 		    break;
 
 		r += r_mb_len;
@@ -653,7 +613,7 @@ const char *revstrstr(const char *haystack, const char *needle, const
 
     return NULL;
 }
-#endif
+#endif /* !NANO_SMALL || !DISABLE_TABCOMP */
 
 #ifndef NANO_SMALL
 /* This function is equivalent to strcasestr(), except in that it scans
@@ -683,7 +643,7 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 	const char *rev_start)
 {
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	char *r_mb = charalloc(MB_CUR_MAX);
 	char *q_mb = charalloc(MB_CUR_MAX);
 	wchar_t wr, wq;
@@ -696,11 +656,14 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 	    int r_mb_len, q_mb_len;
 
 	    while (*q != '\0') {
+		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
+
 		r_mb_len = parse_mbchar(r, r_mb, NULL, NULL);
 
 		if (mbtowc(&wr, r_mb, r_mb_len) <= 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wr = (unsigned char)*r;
+		    bad_r_mb = TRUE;
 		}
 
 		q_mb_len = parse_mbchar(q, q_mb, NULL, NULL);
@@ -708,9 +671,11 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 		if (mbtowc(&wq, q_mb, q_mb_len) <= 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wq = (unsigned char)*q;
+		    bad_q_mb = TRUE;
 		}
 
-		if (towlower(wr) != towlower(wq))
+		if (bad_r_mb != bad_q_mb ||
+			towlower(wr) != towlower(wq))
 		    break;
 
 		r += r_mb_len;
@@ -737,7 +702,7 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 #endif
 	return revstrcasestr(haystack, needle, rev_start);
 }
-#endif
+#endif /* !NANO_SMALL */
 
 /* This function is equivalent to strlen() for multibyte strings. */
 size_t mbstrlen(const char *s)
@@ -766,13 +731,12 @@ size_t mbstrnlen(const char *s, size_t maxlen)
     assert(s != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	size_t n = 0;
-	char *s_mb = charalloc(MB_CUR_MAX);
 	int s_mb_len;
 
 	while (*s != '\0') {
-	    s_mb_len = parse_mbchar(s, s_mb, NULL, NULL);
+	    s_mb_len = parse_mbchar(s, NULL, NULL, NULL);
 
 	    if (maxlen == 0)
 		break;
@@ -782,8 +746,6 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 	    n++;
 	}
 
-	free(s_mb);
-
 	return n;
     } else
 #endif
@@ -791,17 +753,65 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 }
 
 #ifndef DISABLE_JUSTIFY
+#ifdef ENABLE_NANORC
+/* Return TRUE if the string s contains one or more blank characters,
+ * and FALSE otherwise. */
+bool has_blank_chars(const char *s)
+{
+    assert(s != NULL);
+
+    for (; *s != '\0'; s++) {
+	if (isblank(*s))
+	    return TRUE;
+    }
+
+    return FALSE;
+}
+
+/* Return TRUE if the multibyte string s contains one or more blank
+ * multibyte characters, and FALSE otherwise. */
+bool has_blank_mbchars(const char *s)
+{
+    assert(s != NULL);
+
+#ifdef NANO_WIDE
+    if (ISSET(USE_UTF8)) {
+	char *chr_mb = charalloc(MB_CUR_MAX);
+	bool retval = FALSE;
+
+	while (*s != '\0') {
+	    int chr_mb_len;
+
+	    chr_mb_len = parse_mbchar(s, chr_mb, NULL, NULL);
+
+	    if (is_blank_mbchar(chr_mb)) {
+		retval = TRUE;
+		break;
+	    }
+
+	    s += chr_mb_len;
+	}
+
+	free(chr_mb);
+
+	return retval;
+    } else
+#endif
+	return has_blank_chars(s);
+}
+#endif /* ENABLE_NANORC */
+
 /* This function is equivalent to strchr() for multibyte strings. */
 char *mbstrchr(const char *s, char *c)
 {
     assert(s != NULL && c != NULL);
 
 #ifdef NANO_WIDE
-    if (!ISSET(NO_UTF8)) {
+    if (ISSET(USE_UTF8)) {
 	char *s_mb = charalloc(MB_CUR_MAX);
 	const char *q = s;
 	wchar_t ws, wc;
-	int s_mb_len, c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
+	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
 	if (c_mb_len <= 0) {
 	    mbtowc(NULL, NULL, 0);
@@ -809,7 +819,7 @@ char *mbstrchr(const char *s, char *c)
 	}
 
 	while (*s != '\0') {
-	    s_mb_len = parse_mbchar(s, s_mb, NULL, NULL);
+	    int s_mb_len = parse_mbchar(s, s_mb, NULL, NULL);
 
 	    if (mbtowc(&ws, s_mb, s_mb_len) <= 0) {
 		mbtowc(NULL, NULL, 0);
@@ -833,4 +843,21 @@ char *mbstrchr(const char *s, char *c)
 #endif
 	return strchr(s, *c);
 }
+#endif /* !DISABLE_JUSTIFY */
+
+#ifdef ENABLE_NANORC
+/* Check if the string s is a valid multibyte string.  Return TRUE if it
+ * is, and FALSE otherwise. */
+bool is_valid_mbstring(const char *s)
+{
+    assert(s != NULL);
+
+    return 
+#ifdef NANO_WIDE
+	ISSET(USE_UTF8) ?
+	(mbstowcs(NULL, s, 0) != (size_t)-1) :
 #endif
+
+	TRUE;
+}
+#endif /* ENABLE_NANORC */
