@@ -1,4 +1,4 @@
-/* $Id: color.c,v 1.10 2005/03/10 20:55:11 dolorous Exp $ */
+/* $Id: color.c,v 1.13 2005/06/19 21:10:48 dolorous Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -29,7 +30,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include "proto.h"
-#include "nano.h"
 
 #ifdef ENABLE_COLOR
 
@@ -122,7 +122,7 @@ void update_color(void)
     if (colorstrings == NULL && syntaxstr != NULL) {
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
-	    if (strcasecmp(tmpsyntax->desc, syntaxstr) == 0)
+	    if (mbstrcasecmp(tmpsyntax->desc, syntaxstr) == 0)
 		colorstrings = tmpsyntax->color;
 	}
     }
