@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.27 2005/03/26 22:49:46 dolorous Exp $ */
+/* $Id: cut.c,v 1.30 2005/05/15 19:57:16 dolorous Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -28,7 +29,6 @@
 #include <stdio.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 static bool keep_cutbuffer = FALSE;
 	/* Should we keep the contents of the cutbuffer? */
@@ -41,12 +41,15 @@ void cutbuffer_reset(void)
 }
 
 /* If we're not on the magicline, move all the text of the current line,
- * plus the newline at the end, to the cutbuffer. */
+ * plus the newline at the end, to the cutbuffer, and set the current
+ * place we want to where the line used to start. */
 void cut_line(void)
 {
-    if (current->next != NULL)
+    if (current->next != NULL) {
 	move_to_filestruct(&cutbuffer, &cutbottom, current, 0,
 		current->next, 0);
+	placewewant = xplustabs();
+    }
 }
 
 #ifndef NANO_SMALL
