@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.91 2005/03/22 04:50:11 dolorous Exp $ */
+/* $Id: global.c,v 1.126 2005/06/28 06:25:34 dolorous Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -26,7 +27,6 @@
 #include <stdlib.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 /* Global variables */
 
@@ -38,7 +38,6 @@ ssize_t wrap_at = -CHARS_FROM_EOL;	/* Right justified fill value,
 
 char *last_search = NULL;	/* Last string we searched for */
 char *last_replace = NULL;	/* Last replacement string */
-int search_last_line;		/* Is this the last search line? */
 
 unsigned long flags = 0;	/* Our flag containing many options */
 WINDOW *topwin;			/* Top buffer */
@@ -55,7 +54,7 @@ int editwinrows = 0;		/* How many rows long is the edit
 filestruct *current;		/* Current buffer pointer */
 size_t current_x = 0;		/* Current x-coordinate in the edit
 				   window */
-int current_y = 0;		/* Current y-coordinate in the edit
+ssize_t current_y = 0;		/* Current y-coordinate in the edit
 				   window */
 filestruct *fileage = NULL;	/* Our file buffer */
 filestruct *edittop = NULL;	/* Pointer to the top of the edit
@@ -103,7 +102,7 @@ char *backup_dir = NULL;	/* Backup directory. */
 #endif
 
 char *answer = NULL;		/* Answer str to many questions */
-int totlines = 0;		/* Total number of lines in the file */
+size_t totlines = 0;		/* Total number of lines in the file */
 size_t totsize = 0;		/* Total number of characters in the
 				   file */
 size_t placewewant = 0;		/* The column we'd like the cursor
@@ -168,12 +167,15 @@ toggle *toggles = NULL;
 #endif
 
 #ifndef NANO_SMALL
-historyheadtype search_history;
-historyheadtype replace_history;
+filestruct *search_history = NULL;
+filestruct *searchage = NULL;
+filestruct *searchbot = NULL;
+filestruct *replace_history = NULL;
+filestruct *replaceage = NULL;
+filestruct *replacebot = NULL;
 #endif
 
 /* Regular expressions */
-
 #ifdef HAVE_REGEX_H
 regex_t search_regexp;		/* Global to store compiled search regexp */
 regmatch_t regmatches[10];	/* Match positions for parenthetical
@@ -239,6 +241,7 @@ void shortcut_init(bool unjustify)
     const char *cancel_msg = N_("Cancel");
     const char *first_line_msg = N_("First Line");
     const char *last_line_msg = N_("Last Line");
+    const char *refresh_msg = N_("Refresh");
 #ifndef NANO_SMALL
     const char *cut_till_end_msg = N_("CutTillEnd");
 #endif
@@ -249,42 +252,48 @@ void shortcut_init(bool unjustify)
 #endif
 #ifndef NANO_SMALL
     const char *case_sens_msg = N_("Case Sens");
-    const char *direction_msg = N_("Direction");
+    const char *backwards_msg = N_("Backwards");
+#endif
 #ifdef HAVE_REGEX_H
     const char *regexp_msg = N_("Regexp");
 #endif
+#ifndef NANO_SMALL
     const char *history_msg = N_("History");
 #ifdef ENABLE_MULTIBUFFER
     const char *new_buffer_msg = N_("New Buffer");
 #endif
-#endif /* !NANO_SMALL */
+#endif
 #ifndef DISABLE_BROWSER
     const char *to_files_msg = N_("To Files");
 #endif
-
 #ifndef DISABLE_HELP
     const char *nano_help_msg = N_("Invoke the help menu");
     const char *nano_exit_msg =
 #ifdef ENABLE_MULTIBUFFER
-	N_("Close currently loaded file/Exit from nano")
+	N_("Close the current file buffer/Exit from nano")
 #else
    	N_("Exit from nano")
 #endif
 	;
-    const char *nano_writeout_msg = N_("Write the current file to disk");
+    const char *nano_writeout_msg =
+	N_("Write the current file to disk");
     const char *nano_justify_msg = N_("Justify the current paragraph");
     const char *nano_insert_msg =
 	N_("Insert another file into the current one");
-    const char *nano_whereis_msg = N_("Search for text within the editor");
+    const char *nano_whereis_msg =
+	N_("Search for text within the editor");
     const char *nano_prevpage_msg = N_("Move to the previous screen");
     const char *nano_nextpage_msg = N_("Move to the next screen");
     const char *nano_cut_msg =
 	N_("Cut the current line and store it in the cutbuffer");
     const char *nano_uncut_msg =
 	N_("Uncut from the cutbuffer into the current line");
-    const char *nano_cursorpos_msg = N_("Show the position of the cursor");
-    const char *nano_spell_msg = N_("Invoke the spell checker, if available");
-    const char *nano_gotoline_msg = N_("Go to a specific line number");
+    const char *nano_cursorpos_msg =
+	N_("Show the position of the cursor");
+    const char *nano_spell_msg =
+	N_("Invoke the spell checker, if available");
+    const char *nano_gotoline_msg =
+	N_("Go to a specific line number and column number");
     const char *nano_replace_msg = N_("Replace text within the editor");
 #ifndef NANO_SMALL
     const char *nano_mark_msg = N_("Mark text at the cursor position");
@@ -294,10 +303,14 @@ void shortcut_init(bool unjustify)
     const char *nano_nextline_msg = N_("Move to the next line");
     const char *nano_forward_msg = N_("Move forward one character");
     const char *nano_back_msg = N_("Move back one character");
-    const char *nano_home_msg = N_("Move to the beginning of the current line");
-    const char *nano_end_msg = N_("Move to the end of the current line");
-    const char *nano_refresh_msg = N_("Refresh (redraw) the current screen");
-    const char *nano_delete_msg = N_("Delete the character under the cursor");
+    const char *nano_home_msg =
+	N_("Move to the beginning of the current line");
+    const char *nano_end_msg =
+	N_("Move to the end of the current line");
+    const char *nano_refresh_msg =
+	N_("Refresh (redraw) the current screen");
+    const char *nano_delete_msg =
+	N_("Delete the character under the cursor");
     const char *nano_backspace_msg =
 	N_("Delete the character to the left of the cursor");
     const char *nano_tab_msg =
@@ -307,6 +320,8 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_SMALL
     const char *nano_nextword_msg = N_("Move forward one word");
     const char *nano_prevword_msg = N_("Move backward one word");
+    const char *nano_wordcount_msg =
+	N_("Count the number of words in the file");
 #endif
 #ifndef DISABLE_JUSTIFY
     const char *nano_parabegin_msg =
@@ -315,8 +330,10 @@ void shortcut_init(bool unjustify)
 	N_("Go to the end of the current paragraph");
 #endif
 #ifdef ENABLE_MULTIBUFFER
-    const char *nano_openprev_msg = N_("Switch to the previous file buffer");
-    const char *nano_opennext_msg = N_("Switch to the next file buffer");
+    const char *nano_openprev_msg =
+	N_("Switch to the previous file buffer");
+    const char *nano_opennext_msg =
+	N_("Switch to the next file buffer");
 #endif
     const char *nano_verbatim_msg = N_("Insert character(s) verbatim");
 #ifndef NANO_SMALL
@@ -330,20 +347,23 @@ void shortcut_init(bool unjustify)
     const char *nano_bracket_msg = N_("Find other bracket");
 #endif
     const char *nano_cancel_msg = N_("Cancel the current function");
-    const char *nano_firstline_msg = N_("Go to the first line of the file");
-    const char *nano_lastline_msg = N_("Go to the last line of the file");
+    const char *nano_firstline_msg =
+	N_("Go to the first line of the file");
+    const char *nano_lastline_msg =
+	N_("Go to the last line of the file");
 #ifndef NANO_SMALL
     const char *nano_case_msg =
 	N_("Make the current search/replace case (in)sensitive");
     const char *nano_reverse_msg =
 	N_("Make the current search/replace go backwards");
+#endif
 #ifdef HAVE_REGEX_H
     const char *nano_regexp_msg = N_("Use regular expressions");
 #endif
-    const char *nano_editstr_msg =
+#ifndef NANO_SMALL
+    const char *nano_history_msg =
 	N_("Edit the previous search/replace strings");
-#endif /* !NANO_SMALL */
-
+#endif
 #ifndef DISABLE_BROWSER
     const char *nano_tofiles_msg = N_("Go to file browser");
 #endif
@@ -354,7 +374,8 @@ void shortcut_init(bool unjustify)
     const char *nano_append_msg = N_("Append to the current file");
     const char *nano_prepend_msg = N_("Prepend to the current file");
 #ifndef NANO_SMALL
-    const char *nano_backup_msg = N_("Back up original file when saving");
+    const char *nano_backup_msg =
+	N_("Back up original file when saving");
     const char *nano_execute_msg = N_("Execute external command");
 #endif
 #if !defined(NANO_SMALL) && defined(ENABLE_MULTIBUFFER)
@@ -480,7 +501,8 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&main_list, NANO_GOTOLINE_KEY, go_to_line_msg,
 	IFHELP(nano_gotoline_msg, NANO_GOTOLINE_ALTKEY),
-	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW, do_gotoline_void);
+	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW,
+	do_gotolinecolumn_void);
 
     sc_init_one(&main_list, NANO_REPLACE_KEY, replace_msg,
 	IFHELP(nano_replace_msg, NANO_ALT_REPLACE_KEY),
@@ -489,7 +511,7 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_SMALL
     sc_init_one(&main_list, NANO_MARK_KEY, N_("Mark Text"),
 	IFHELP(nano_mark_msg, NANO_MARK_ALTKEY), NANO_MARK_FKEY,
-	NANO_NO_KEY, NOVIEW, do_mark);
+	NANO_NO_KEY, VIEW, do_mark);
 
     sc_init_one(&main_list, NANO_NO_KEY, N_("Where Is Next"),
 	IFHELP(nano_whereis_next_msg, NANO_WHEREIS_NEXT_KEY),
@@ -520,7 +542,7 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_end_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, do_end);
 
-    sc_init_one(&main_list, NANO_REFRESH_KEY, N_("Refresh"),
+    sc_init_one(&main_list, NANO_REFRESH_KEY, refresh_msg,
 	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, total_refresh);
 
@@ -543,11 +565,15 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_SMALL
     sc_init_one(&main_list, NANO_NEXTWORD_KEY, N_("Next Word"),
 	IFHELP(nano_nextword_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_next_word);
+	NANO_NO_KEY, VIEW, do_next_word_void);
 
     sc_init_one(&main_list, NANO_NO_KEY, N_("Prev Word"),
 	IFHELP(nano_prevword_msg, NANO_PREVWORD_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_prev_word);
+	NANO_NO_KEY, VIEW, do_prev_word_void);
+
+    sc_init_one(&main_list, NANO_NO_KEY, N_("Word Count"),
+	IFHELP(nano_wordcount_msg, NANO_WORDCOUNT_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, do_word_count);
 #endif
 
 #ifndef DISABLE_JUSTIFY
@@ -652,27 +678,29 @@ void shortcut_init(bool unjustify)
 	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_NO_KEY, direction_msg,
+    sc_init_one(&whereis_list, NANO_NO_KEY, backwards_msg,
 	IFHELP(nano_reverse_msg, TOGGLE_BACKWARDS_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
+#endif
 
 #ifdef HAVE_REGEX_H
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_NO_KEY, regexp_msg,
-	IFHELP(nano_regexp_msg, TOGGLE_REGEXP_KEY), NANO_NO_KEY,
+	IFHELP(nano_regexp_msg, NANO_REGEXP_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
 #endif
 
+#ifndef NANO_SMALL
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_HISTORY_KEY, history_msg,
-	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
+    sc_init_one(&whereis_list, NANO_PREVLINE_KEY, history_msg,
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_CUTTILLEND_KEY, cut_till_end_msg,
 	IFHELP(nano_cut_till_end_msg, NANO_CUTTILLEND_ALTKEY),
 	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_cut_till_end);
-#endif /* !NANO_SMALL */
+#endif
 
 #ifndef DISABLE_JUSTIFY
     /* Translators: try to keep this string under 10 characters long */
@@ -719,20 +747,22 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_case_msg, TOGGLE_CASE_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
 
-    sc_init_one(&replace_list, NANO_NO_KEY, direction_msg,
+    sc_init_one(&replace_list, NANO_NO_KEY, backwards_msg,
 	IFHELP(nano_reverse_msg, TOGGLE_BACKWARDS_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
-
-#ifdef HAVE_REGEX_H
-    sc_init_one(&replace_list, NANO_NO_KEY, regexp_msg,
-	IFHELP(nano_regexp_msg, TOGGLE_REGEXP_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
 #endif
 
-    sc_init_one(&replace_list, NANO_HISTORY_KEY, history_msg,
-	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
+#ifdef HAVE_REGEX_H
+    sc_init_one(&replace_list, NANO_NO_KEY, regexp_msg,
+	IFHELP(nano_regexp_msg, NANO_REGEXP_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
-#endif /* !NANO_SMALL */
+#endif
+
+#ifndef NANO_SMALL
+    sc_init_one(&replace_list, NANO_PREVLINE_KEY, history_msg,
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, NULL);
+#endif
 
     free_shortcutage(&replace_list_2);
 
@@ -759,8 +789,8 @@ void shortcut_init(bool unjustify)
 	NANO_NO_KEY, VIEW, do_last_line);
 
 #ifndef NANO_SMALL
-    sc_init_one(&replace_list_2, NANO_HISTORY_KEY, history_msg,
-	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
+    sc_init_one(&replace_list_2, NANO_PREVLINE_KEY, history_msg,
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
 #endif
 
@@ -795,6 +825,14 @@ void shortcut_init(bool unjustify)
 #ifndef DISABLE_HELP
     free_shortcutage(&help_list);
 
+    sc_init_one(&help_list, NANO_REFRESH_KEY, refresh_msg,
+	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, NULL);
+
+    sc_init_one(&help_list, NANO_EXIT_KEY, exit_msg,
+	IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
+	NANO_NO_KEY, VIEW, NULL);
+
     sc_init_one(&help_list, NANO_PREVPAGE_KEY, prev_page_msg,
 	IFHELP(nano_prevpage_msg, NANO_NO_KEY), NANO_PREVPAGE_FKEY,
 	NANO_NO_KEY, VIEW, NULL);
@@ -810,14 +848,6 @@ void shortcut_init(bool unjustify)
     sc_init_one(&help_list, NANO_NEXTLINE_KEY, N_("Next Line"),
 	IFHELP(nano_nextline_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, NULL);
-
-    sc_init_one(&help_list, NANO_REFRESH_KEY, N_("Refresh"),
-	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
-
-    sc_init_one(&help_list, NANO_EXIT_KEY, exit_msg,
-	IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-	NANO_NO_KEY, VIEW, NULL);
 #endif
 
     free_shortcutage(&writefile_list);
@@ -1002,10 +1032,6 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_nextpage_msg, NANO_NO_KEY), NANO_NEXTPAGE_FKEY,
 	NANO_NO_KEY, VIEW, NULL);
 
-    sc_init_one(&browser_list, NANO_REFRESH_KEY, N_("Refresh"),
-	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
-
     /* Translators: try to keep this string under 22 characters long */
     sc_init_one(&browser_list, NANO_GOTOLINE_KEY, N_("Go To Dir"),
 	IFHELP(nano_gotodir_msg, NANO_GOTOLINE_ALTKEY),
@@ -1039,6 +1065,7 @@ void shortcut_init(bool unjustify)
 void free_shortcutage(shortcut **shortcutage)
 {
     assert(shortcutage != NULL);
+
     while (*shortcutage != NULL) {
 	shortcut *ps = *shortcutage;
 	*shortcutage = (*shortcutage)->next;
@@ -1077,28 +1104,33 @@ void toggle_init(void)
 	return;
 
     toggle_init_one(TOGGLE_NOHELP_KEY, N_("Help mode"), NO_HELP);
+    toggle_init_one(TOGGLE_MORESPACE_KEY,
+	N_("Use of more space for editing"), MORE_SPACE);
 #ifdef ENABLE_MULTIBUFFER
     /* If we're using restricted mode, the multibuffer toggle is
      * disabled.  It's useless since inserting files is disabled. */
     if (!ISSET(RESTRICTED))
 	toggle_init_one(TOGGLE_MULTIBUFFER_KEY,
 		N_("Multiple file buffers"), MULTIBUFFER);
-#endif
-    toggle_init_one(TOGGLE_CONST_KEY, N_("Constant cursor position"),
-	CONSTUPDATE);
-    toggle_init_one(TOGGLE_AUTOINDENT_KEY, N_("Auto indent"),
-	AUTOINDENT);
-#ifndef DISABLE_WRAPPING
-    toggle_init_one(TOGGLE_WRAP_KEY, N_("Auto line wrap"), NO_WRAP);
 #endif
     toggle_init_one(TOGGLE_CUTTOEND_KEY, N_("Cut to end"), CUT_TO_END);
+#ifndef DISABLE_WRAPPING
+    toggle_init_one(TOGGLE_WRAP_KEY, N_("Long line wrapping"),
+	NO_WRAP);
+#endif
+#ifndef DISABLE_MOUSE
+    toggle_init_one(TOGGLE_MOUSE_KEY, N_("Mouse support"), USE_MOUSE);
+#endif
     /* If we're using restricted mode, the suspend toggle is disabled.
      * It's useless since suspending is disabled. */
     if (!ISSET(RESTRICTED))
 	toggle_init_one(TOGGLE_SUSPEND_KEY, N_("Suspend"), SUSPEND);
-#ifndef DISABLE_MOUSE
-    toggle_init_one(TOGGLE_MOUSE_KEY, N_("Mouse support"), USE_MOUSE);
-#endif
+    toggle_init_one(TOGGLE_CONST_KEY,
+	N_("Constant cursor position display"), CONST_UPDATE);
+    toggle_init_one(TOGGLE_AUTOINDENT_KEY, N_("Auto indent"),
+	AUTOINDENT);
+    toggle_init_one(TOGGLE_TABSTOSPACES_KEY,
+	N_("Conversion of typed tabs to spaces"), TABS_TO_SPACES);
     /* If we're using restricted mode, the DOS/Mac conversion toggle is
      * disabled.  It's useless since inserting files is disabled. */
     if (!ISSET(RESTRICTED))
@@ -1110,7 +1142,7 @@ void toggle_init(void)
 	toggle_init_one(TOGGLE_BACKUP_KEY, N_("Backup files"),
 		BACKUP_FILE);
     toggle_init_one(TOGGLE_SMOOTH_KEY, N_("Smooth scrolling"),
-	SMOOTHSCROLL);
+	SMOOTH_SCROLL);
     toggle_init_one(TOGGLE_SMARTHOME_KEY, N_("Smart home key"),
 	SMART_HOME);
 #ifdef ENABLE_COLOR
@@ -1121,22 +1153,7 @@ void toggle_init(void)
     toggle_init_one(TOGGLE_WHITESPACE_KEY, N_("Whitespace display"),
 	WHITESPACE_DISPLAY);
 #endif
-    toggle_init_one(TOGGLE_MORESPACE_KEY, N_("Use of more space for editing"),
-	MORE_SPACE);
 }
-
-#ifdef DEBUG
-/* Deallocate all of the toggles. */
-void free_toggles(void)
-{
-    while (toggles != NULL) {
-	toggle *pt = toggles;	/* Think "previous toggle". */
-
-	toggles = toggles->next;
-	free(pt);
-    }
-}
-#endif
 #endif /* !NANO_SMALL */
 
 /* This function is called just before calling exit().  Practically, the
@@ -1156,7 +1173,8 @@ void thanks_for_all_the_fish(void)
 	free(quotestr);
 #ifdef HAVE_REGEX_H
     regfree(&quotereg);
-    free(quoteerr);
+    if (quoteerr != NULL)
+	free(quoteerr);
 #endif
 #endif
 #ifndef NANO_SMALL
@@ -1213,24 +1231,31 @@ void thanks_for_all_the_fish(void)
     free_shortcutage(&browser_list);
     free_shortcutage(&gotodir_list);
 #endif
-
 #ifndef NANO_SMALL
-    free_toggles();
-#endif
+    /* Free the memory associated with each toggle. */
+    while (toggles != NULL) {
+	toggle *t = toggles;
 
+	toggles = toggles->next;
+	free(t);
+    }
+#endif
 #ifdef ENABLE_MULTIBUFFER
+    /* Free the memory associated with each open file buffer. */
     if (open_files != NULL) {
-	/* We free the memory associated with each open file. */
-	while (open_files->prev != NULL)
-	    open_files = open_files->prev;
+	/* Make sure open_files->fileage is up to date, in case we've
+	 * cut the top line of the file. */
+	open_files->fileage = fileage;
+
 	free_openfilestruct(open_files);
     }
 #else
-    free_filestruct(fileage);
+    if (fileage != NULL)
+	free_filestruct(fileage);
 #endif
-
 #ifdef ENABLE_COLOR
-    free(syntaxstr);
+    if (syntaxstr != NULL)
+	free(syntaxstr);
     while (syntaxes != NULL) {
 	syntaxtype *bill = syntaxes;
 
@@ -1257,12 +1282,15 @@ void thanks_for_all_the_fish(void)
     }
 #endif /* ENABLE_COLOR */
 #ifndef NANO_SMALL
-    /* free history lists */
-    free_history(&search_history);
-    free_history(&replace_history);
+    /* Free the search and replace history lists. */
+    if (searchage != NULL)
+	free_filestruct(searchage);
+    if (replaceage != NULL)
+	free_filestruct(replaceage);
 #endif
 #ifdef ENABLE_NANORC
-    free(homedir);
+    if (homedir != NULL)
+	free(homedir);
 #endif
 }
 #endif /* DEBUG */
