@@ -1,4 +1,4 @@
-/* $Id: move.c,v 1.26 2005/03/09 20:35:10 dolorous Exp $ */
+/* $Id: move.c,v 1.30 2005/06/28 18:57:28 dolorous Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -28,7 +29,6 @@
 #include <ctype.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 void do_first_line(void)
 {
@@ -107,7 +107,7 @@ void do_page_up(void)
 	/* If we're in smooth scrolling mode and there's at least one
 	 * page of text left, move the current line of the edit window
 	 * up a page. */
-	if (ISSET(SMOOTHSCROLL) && current->lineno > editwinrows - 2) {
+	if (ISSET(SMOOTH_SCROLL) && current->lineno > editwinrows - 2) {
 	    int i;
 	    for (i = 0; i < editwinrows - 2; i++)
 		current = current->prev;
@@ -153,7 +153,7 @@ void do_page_down(void)
 	/* If we're in smooth scrolling mode and there's at least one
 	 * page of text left, move the current line of the edit window
 	 * down a page. */
-	if (ISSET(SMOOTHSCROLL) && current->lineno + editwinrows - 2 <=
+	if (ISSET(SMOOTH_SCROLL) && current->lineno + editwinrows - 2 <=
 		filebot->lineno) {
 	    int i;
 	    for (i = 0; i < editwinrows - 2; i++)
@@ -201,7 +201,7 @@ void do_up(void)
     if (current_y == 0)
 	edit_scroll(UP,
 #ifndef NANO_SMALL
-		ISSET(SMOOTHSCROLL) ? 1 :
+		ISSET(SMOOTH_SCROLL) ? 1 :
 #endif
 		editwinrows / 2);
 
@@ -225,6 +225,7 @@ void do_down(void)
 	return;
 
     assert(current_y == current->lineno - edittop->lineno);
+
     current = current->next;
     current_x = actual_x(current->data, placewewant);
 
@@ -234,7 +235,7 @@ void do_down(void)
     if (current_y == editwinrows - 1)
 	edit_scroll(DOWN,
 #ifndef NANO_SMALL
-		ISSET(SMOOTHSCROLL) ? 1 :
+		ISSET(SMOOTH_SCROLL) ? 1 :
 #endif
 		editwinrows / 2);
 
