@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.90 2005/03/26 04:42:28 dolorous Exp $ */
+/* $Id: nano.h,v 1.117 2005/06/28 06:25:34 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -8,22 +8,27 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
 #ifndef NANO_H
 #define NANO_H 1
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #ifdef __TANDEM
-/* Tandem NonStop Kernel */
+/* Tandem NonStop Kernel. */
 #include <floss.h>
 #define NANO_ROOT_UID 65535
 #else
@@ -34,17 +39,17 @@
 #include <limits.h>
 #endif
 
-/* Macros for the flags long. */
+/* Macros for flags. */
 #define SET(bit) flags |= bit
 #define UNSET(bit) flags &= ~bit
 #define ISSET(bit) ((flags & bit) != 0)
 #define TOGGLE(bit) flags ^= bit
 
-/* Macros for character allocation. */
+/* Macros for character allocation, etc. */
 #define charalloc(howmuch) (char *)nmalloc((howmuch) * sizeof(char))
 #define charealloc(ptr, howmuch) (char *)nrealloc(ptr, (howmuch) * sizeof(char))
 #define charmove(dest, src, n) memmove(dest, src, (n) * sizeof(char))
-#define charcpy(dest, src, n) memcpy(dest, src, (n) * sizeof(char))
+#define charset(dest, src, n) memset(dest, src, (n) * sizeof(char))
 
 /* Other macros. */
 #ifdef BROKEN_REGEXEC
@@ -52,8 +57,7 @@
 #define regexec(preg, string, nmatch, pmatch, eflags) safe_regexec(preg, string, nmatch, pmatch, eflags)
 #endif
 
-/* Set a default value for PATH_MAX, so we can use it in lines like
- * "path = getcwd(NULL, PATH_MAX + 1);". */
+/* Set a default value for PATH_MAX if there isn't one. */
 #ifndef PATH_MAX
 #define PATH_MAX 4096
 #endif
@@ -88,9 +92,8 @@
 #include <stddef.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "config.h"
 
-/* If no snprintf() or vsnprintf(), use the versions from glib. */
+/* If no snprintf() or vsnprintf(), use the versions from glib 2.x. */
 #if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
 #include <glib.h>
 #ifndef HAVE_SNPRINTF
@@ -101,8 +104,15 @@
 #endif
 #endif
 
-/* If no strcasecmp(), strncasecmp(), strcasestr(), strnlen(),
- * getdelim(), or getline(), use the versions we have. */
+/* If no isblank(), iswblank(), strcasecmp(), strncasecmp(),
+ * strcasestr(), strnlen(), getdelim(), or getline(), use the versions
+ * we have. */
+#ifndef HAVE_ISBLANK
+#define isblank nisblank
+#endif
+#ifndef HAVE_ISWBLANK
+#define iswblank niswblank
+#endif
 #ifndef HAVE_STRCASECMP
 #define strcasecmp nstrcasecmp
 #endif
@@ -152,7 +162,7 @@ typedef struct filestruct {
     char *data;
     struct filestruct *next;	/* Next node. */
     struct filestruct *prev;	/* Previous node. */
-    int lineno;			/* The line number. */
+    ssize_t lineno;		/* The line number. */
 } filestruct;
 
 #ifdef ENABLE_MULTIBUFFER
@@ -179,7 +189,7 @@ typedef struct openfilestruct {
     size_t current_x;		/* Current file's x-coordinate
 				 * position. */
     size_t placewewant;		/* Current file's place we want. */
-    int totlines;		/* Current file's total number of
+    size_t totlines;		/* Current file's total number of
 				 * lines. */
     size_t totsize;		/* Current file's total size. */
     unsigned long flags;	/* Current file's flags: modification
@@ -237,7 +247,7 @@ typedef struct rcoption {
 typedef struct colortype {
     int fg;			/* Foreground color. */
     int bg;			/* Background color. */
-    int bright;			/* Is this color A_BOLD? */
+    bool bright;		/* Is this color A_BOLD? */
     int pairnum;		/* Color pair number used for this
 				 * foreground/background. */
     regex_t start;		/* Start (or all) of the regex
@@ -261,31 +271,12 @@ typedef struct syntaxtype {
 } syntaxtype;
 #endif
 
-#ifndef NANO_SMALL
-typedef struct historytype {
-    struct historytype *next;
-    struct historytype *prev;
-    char *data;
-} historytype;
-
-typedef struct historyheadtype {
-    struct historytype *next;	/* Keep *next and *prev members
-				 * together. */
-    struct historytype *prev;	/* And in the same order as in
-				 * historytype. */
-    struct historytype *tail;
-    struct historytype *current;
-    int count;
-    int len;
-} historyheadtype;
-#endif
-
-/* Bitwise flags so we can save space (or more correctly, not waste
- * it). */
+/* Bitwise flags so that we can save space (or, more correctly, not
+ * waste it). */
 #define MODIFIED		(1<<0)
 #define CASE_SENSITIVE		(1<<1)
 #define MARK_ISSET		(1<<2)
-#define CONSTUPDATE		(1<<3)
+#define CONST_UPDATE		(1<<3)
 #define NO_HELP			(1<<4)
 #define NOFOLLOW_SYMLINKS	(1<<5)
 #define SUSPEND			(1<<6)
@@ -296,25 +287,25 @@ typedef struct historyheadtype {
 #define USE_REGEXP		(1<<11)
 #define TEMP_FILE		(1<<12)
 #define CUT_TO_END		(1<<13)
-#define REVERSE_SEARCH		(1<<14)
+#define BACKWARDS_SEARCH	(1<<14)
 #define MULTIBUFFER		(1<<15)
-#define SMOOTHSCROLL		(1<<16)
-#define DISABLE_CURPOS		(1<<17)	/* Damn, we still need it. */
-#define REBIND_DELETE		(1<<18)
-#define NO_CONVERT		(1<<19)
-#define BACKUP_FILE		(1<<20)
-#define NO_RCFILE		(1<<21)
-#define NO_COLOR_SYNTAX		(1<<22)
-#define PRESERVE		(1<<23)
-#define HISTORY_CHANGED		(1<<24)
-#define HISTORYLOG		(1<<25)
-#define RESTRICTED		(1<<26)
-#define SMART_HOME		(1<<27)
-#define WHITESPACE_DISPLAY	(1<<28)
-#define MORE_SPACE		(1<<29)
-#define NO_UTF8			(1<<30)
+#define SMOOTH_SCROLL		(1<<16)
+#define REBIND_DELETE		(1<<17)
+#define NO_CONVERT		(1<<18)
+#define BACKUP_FILE		(1<<19)
+#define NO_RCFILE		(1<<20)
+#define NO_COLOR_SYNTAX		(1<<21)
+#define PRESERVE		(1<<22)
+#define HISTORYLOG		(1<<23)
+#define RESTRICTED		(1<<24)
+#define SMART_HOME		(1<<25)
+#define WHITESPACE_DISPLAY	(1<<26)
+#define MORE_SPACE		(1<<27)
+#define TABS_TO_SPACES		(1<<28)
+#define QUICK_BLANK		(1<<29)
+#define USE_UTF8		(1<<30)
 
-/* Control key sequences, changing these would be very very bad. */
+/* Control key sequences.  Changing these would be very, very bad. */
 #define NANO_CONTROL_SPACE 0
 #define NANO_CONTROL_A 1
 #define NANO_CONTROL_B 2
@@ -392,9 +383,6 @@ typedef struct historyheadtype {
 /* No key at all. */
 #define NANO_NO_KEY		-2
 
-/* Special sentinel key used for search/replace history. */
-#define NANO_HISTORY_KEY	-3
-
 /* Normal keys. */
 #define NANO_XON_KEY		NANO_CONTROL_Q
 #define NANO_XOFF_KEY		NANO_CONTROL_S
@@ -417,6 +405,7 @@ typedef struct historyheadtype {
 #define NANO_WHEREIS_NEXT_KEY	NANO_ALT_W
 #define NANO_WHEREIS_NEXT_FKEY	KEY_F(16)
 #define NANO_TOOTHERWHEREIS_KEY	NANO_CONTROL_T
+#define NANO_REGEXP_KEY		NANO_ALT_R
 #define NANO_REPLACE_KEY	NANO_CONTROL_4
 #define NANO_REPLACE_FKEY	KEY_F(14)
 #define NANO_ALT_REPLACE_KEY	NANO_ALT_R
@@ -466,6 +455,7 @@ typedef struct historyheadtype {
 #define NANO_BRACKET_KEY	NANO_ALT_RBRACKET
 #define NANO_NEXTWORD_KEY	NANO_CONTROL_SPACE
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
+#define NANO_WORDCOUNT_KEY	NANO_ALT_D
 #define NANO_CUTTILLEND_KEY	NANO_CONTROL_X
 #define NANO_CUTTILLEND_ALTKEY	NANO_ALT_T
 #define NANO_PARABEGIN_KEY	NANO_CONTROL_W
@@ -486,7 +476,6 @@ typedef struct historyheadtype {
 #define TOGGLE_NOHELP_KEY	NANO_ALT_X
 #define TOGGLE_MOUSE_KEY	NANO_ALT_M
 #define TOGGLE_CUTTOEND_KEY	NANO_ALT_K
-#define TOGGLE_REGEXP_KEY	NANO_ALT_R
 #define TOGGLE_WRAP_KEY		NANO_ALT_L
 #define TOGGLE_BACKWARDS_KEY	NANO_ALT_B
 #define TOGGLE_CASE_KEY		NANO_ALT_C
@@ -500,6 +489,7 @@ typedef struct historyheadtype {
 #define TOGGLE_SMARTHOME_KEY	NANO_ALT_H
 #define TOGGLE_WHITESPACE_KEY	NANO_ALT_P
 #define TOGGLE_MORESPACE_KEY	NANO_ALT_O
+#define TOGGLE_TABSTOSPACES_KEY	NANO_ALT_Q
 #endif /* !NANO_SMALL */
 
 #define MAIN_VISIBLE 12
@@ -517,7 +507,11 @@ typedef struct historyheadtype {
 /* Default width of a tab. */
 #define WIDTH_OF_TAB 8
 
-/* Maximum number of search/replace history strings saved. */
+/* Maximum number of search/replace history strings saved, not counting
+ * the blank lines at their ends. */
 #define MAX_SEARCH_HISTORY 100
 
+/* Maximum number of bytes we read from a file at one time. */
+#define MAX_BUF_SIZE 128
+
 #endif /* !NANO_H */
