@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.185 2005/03/26 22:49:46 dolorous Exp $ */
+/* $Id: proto.h,v 1.238 2005/06/30 03:55:55 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -8,17 +8,21 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
+#ifndef PROTO_H
+#define PROTO_H 1
+
 /* Externs. */
 
 #include <sys/types.h>
@@ -34,8 +38,8 @@ extern ssize_t wrap_at;
 #endif
 extern int editwinrows;
 extern size_t current_x;
-extern int current_y;
-extern int totlines;
+extern ssize_t current_y;
+extern size_t totlines;
 extern size_t placewewant;
 #ifndef NANO_SMALL
 extern size_t mark_beginx;
@@ -140,44 +144,45 @@ extern toggle *toggles;
 #endif
 
 #ifndef NANO_SMALL
-extern historyheadtype search_history;
-extern historyheadtype replace_history;
+extern filestruct *search_history;
+extern filestruct *searchage;
+extern filestruct *searchbot;
+extern filestruct *replace_history;
+extern filestruct *replaceage;
+extern filestruct *replacebot;
 #endif
 
 extern bool curses_ended;
 
 extern char *homedir;
 
-/* Functions we want available. */
+/* The functions we want available. */
 
 /* Public functions in chars.c. */
+#ifndef HAVE_ISBLANK
+bool nisblank(int c);
+#endif
+#if !defined(HAVE_ISWBLANK) && defined(NANO_WIDE)
+bool niswblank(wchar_t wc);
+#endif
 bool is_byte(int c);
-bool is_alnum_char(int c);
 bool is_alnum_mbchar(const char *c);
-#ifdef NANO_WIDE
-bool is_alnum_wchar(wchar_t wc);
-#endif
-bool is_blank_char(int c);
 bool is_blank_mbchar(const char *c);
-#ifdef NANO_WIDE
-bool is_blank_wchar(wchar_t wc);
-#endif
 bool is_cntrl_char(int c);
-bool is_cntrl_mbchar(const char *c);
 #ifdef NANO_WIDE
 bool is_cntrl_wchar(wchar_t wc);
 #endif
-unsigned char control_rep(unsigned char c);
-char *control_mbrep(const char *c, char *crep, int *crep_len);
+bool is_cntrl_mbchar(const char *c);
+bool is_punct_mbchar(const char *c);
+bool is_word_mbchar(const char *c, bool allow_punct);
+char control_rep(char c);
 #ifdef NANO_WIDE
 wchar_t control_wrep(wchar_t c);
 #endif
+char *control_mbrep(const char *c, char *crep, int *crep_len);
 int mbwidth(const char *c);
 int mb_cur_max(void);
 char *make_mbchar(int chr, int *chr_mb_len);
-#if defined(ENABLE_NANORC) || defined(ENABLE_EXTRA)
-char *make_mbstring(const char *str);
-#endif
 int parse_mbchar(const char *buf, char *chr, bool *bad_chr, size_t
 	*col);
 size_t move_mbleft(const char *buf, size_t pos);
@@ -194,9 +199,11 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 const char *nstrcasestr(const char *haystack, const char *needle);
 #endif
 const char *mbstrcasestr(const char *haystack, const char *needle);
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) || !defined(DISABLE_TABCOMP)
 const char *revstrstr(const char *haystack, const char *needle, const
 	char *rev_start);
+#endif
+#ifndef NANO_SMALL
 const char *revstrcasestr(const char *haystack, const char *needle,
 	const char *rev_start);
 const char *mbrevstrcasestr(const char *haystack, const char *needle,
@@ -208,8 +215,15 @@ size_t nstrnlen(const char *s, size_t maxlen);
 #endif
 size_t mbstrnlen(const char *s, size_t maxlen);
 #ifndef DISABLE_JUSTIFY
+#ifdef ENABLE_NANORC
+bool has_blank_chars(const char *s);
+bool has_blank_mbchars(const char *s);
+#endif
 char *mbstrchr(const char *s, char *c);
 #endif
+#ifdef ENABLE_NANORC
+bool is_valid_mbstring(const char *s);
+#endif
 
 /* Public functions in color.c. */
 #ifdef ENABLE_COLOR
@@ -233,12 +247,12 @@ void do_uncut_text(void);
 
 /* Public functions in files.c. */
 void new_file(void);
-filestruct *read_line(char *buf, filestruct *prev, bool *first_line_ins,
-	size_t len);
+filestruct *read_line(char *buf, filestruct *prevnode, bool
+	*first_line_ins, size_t buf_len);
 void load_file(void);
 void read_file(FILE *f, const char *filename);
 int open_file(const char *filename, bool newfie, FILE **f);
-char *get_next_filename(const char *name);
+char *get_next_filename(const char *name, const char *suffix);
 #ifndef NANO_SMALL
 void execute_command(const char *command);
 #endif
@@ -262,18 +276,14 @@ void free_openfilestruct(openfilestruct *src);
 #endif
 void add_open_file(bool update);
 void load_open_file(void);
-void open_prevnext_file(bool next);
+void open_prevnext_file(bool next_file);
 void open_prevfile_void(void);
 void open_nextfile_void(void);
 bool close_open_file(void);
 #endif
-#if !defined(DISABLE_SPELLER) || !defined(DISABLE_OPERATINGDIR)
 char *get_full_path(const char *origpath);
-#endif
-#ifndef DISABLE_SPELLER
 char *check_writable_directory(const char *path);
-char *safe_tempnam(void);
-#endif
+char *safe_tempfile(FILE **f);
 #ifndef DISABLE_OPERATINGDIR
 void init_operating_dir(void);
 bool check_operating_dir(const char *currpath, bool allow_tabcomp);
@@ -282,16 +292,17 @@ bool check_operating_dir(const char *currpath, bool allow_tabcomp);
 void init_backup_dir(void);
 #endif
 int copy_file(FILE *inn, FILE *out);
-int write_file(const char *name, bool tmp, int append, bool
-	nonamechange);
+int write_file(const char *name, FILE *f_open, bool tmp, int append,
+	bool nonamechange);
 #ifndef NANO_SMALL
-int write_marked(const char *name, bool tmp, int append);
+int write_marked(const char *name, FILE *f_open, bool tmp, int append);
 #endif
 int do_writeout(bool exiting);
 void do_writeout_void(void);
 char *real_dir_from_tilde(const char *buf);
 #if !defined(DISABLE_TABCOMP) || !defined(DISABLE_BROWSER)
 int diralphasort(const void *va, const void *vb);
+void free_chararray(char **array, size_t len);
 #endif
 #ifndef DISABLE_TABCOMP
 char **username_tab_completion(const char *buf, size_t *num_matches,
@@ -302,7 +313,6 @@ char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list);
 #endif
 const char *tail(const char *foo);
 #ifndef DISABLE_BROWSER
-void free_charptrarray(char **array, size_t len);
 void striponedir(char *path);
 char **browser_init(const char *path, int *longest, size_t *numents, DIR
 	*dir);
@@ -312,7 +322,7 @@ char *do_browse_from(const char *inpath);
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
 char *histfilename(void);
 void load_history(void);
-bool writehist(FILE *hist, historyheadtype *histhead);
+bool writehist(FILE *hist, filestruct *histhead);
 void save_history(void);
 #endif
 
@@ -321,9 +331,6 @@ size_t length_of_list(const shortcut *s);
 #ifndef NANO_SMALL
 void toggle_init_one(int val, const char *desc, long flag);
 void toggle_init(void);
-#ifdef DEBUG
-void free_toggles(void);
-#endif
 #endif
 void sc_init_one(shortcut **shortcutage, int key, const char *desc,
 #ifndef DISABLE_HELP
@@ -402,8 +409,11 @@ void do_delete(void);
 void do_tab(void);
 void do_enter(void);
 #ifndef NANO_SMALL
-void do_next_word(void);
-void do_prev_word(void);
+bool do_next_word(bool allow_punct, bool allow_update);
+void do_next_word_void(void);
+bool do_prev_word(bool allow_punct, bool allow_update);
+void do_prev_word_void(void);
+void do_word_count(void);
 void do_mark(void);
 #endif
 #ifndef DISABLE_WRAPPING
@@ -454,7 +464,7 @@ void allow_pending_sigwinch(bool allow);
 #ifndef NANO_SMALL
 void do_toggle(const toggle *which);
 #endif
-void disable_extended_input(void);
+void disable_extended_io(void);
 void disable_signals(void);
 #ifndef NANO_SMALL
 void enable_signals(void);
@@ -479,7 +489,7 @@ int color_to_int(const char *colorname, bool *bright);
 char *parse_next_regex(char *ptr);
 bool nregcomp(regex_t *preg, const char *regex, int eflags);
 void parse_syntax(char *ptr);
-void parse_colors(char *ptr);
+void parse_colors(char *ptr, bool icase);
 #endif /* ENABLE_COLOR */
 void parse_rcfile(FILE *rcstream);
 void do_rcfile(void);
@@ -505,32 +515,37 @@ void do_research(void);
 #endif
 void replace_abort(void);
 #ifdef HAVE_REGEX_H
-int replace_regexp(char *string, bool create_flag);
+int replace_regexp(char *string, bool create);
 #endif
 char *replace_line(const char *needle);
 ssize_t do_replace_loop(const char *needle, const filestruct
 	*real_current, size_t *real_current_x, bool wholewords, bool
 	*canceled);
 void do_replace(void);
-void do_gotoline(int line, bool save_pos);
-void do_gotoline_void(void);
+void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
+	bool interactive, bool save_pos);
+void do_gotolinecolumn_void(void);
 #if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
-void do_gotopos(int line, size_t pos_x, int pos_y, size_t pos_pww);
+void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
+	pos_pww);
 #endif
-void do_find_bracket(void);
 #ifndef NANO_SMALL
+#ifdef HAVE_REGEX_H
+void do_find_bracket(void);
+#endif
+#ifdef ENABLE_NANORC
+bool history_has_changed(void);
+#endif
 void history_init(void);
-historytype *find_node(historytype *h, const char *s);
-void remove_node(historytype *r);
-void insert_node(historytype *h, const char *s);
-void update_history(historyheadtype *h, const char *s);
-char *get_history_older(historyheadtype *h);
-char *get_history_newer(historyheadtype *h);
-char *get_history_completion(historyheadtype *h, char *s);
-#ifdef DEBUG
-void free_history(historyheadtype *h);
-#endif
+filestruct *find_history(filestruct *h_start, filestruct *h_end, const
+	char *s, size_t len);
+void update_history(filestruct **h, const char *s);
+char *get_history_older(filestruct **h);
+char *get_history_newer(filestruct **h);
+#ifndef DISABLE_TABCOMP
+char *get_history_completion(filestruct **h, char *s, size_t len);
 #endif
+#endif /* !NANO_SMALL */
 
 /* Public functions in utils.c. */
 #ifdef HAVE_REGEX_H
@@ -540,10 +555,11 @@ int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
 #endif
 int regexp_bol_or_eol(const regex_t *preg, const char *string);
 #endif
-int num_of_digits(int n);
+int digits(size_t n);
 void get_homedir(void);
 bool parse_num(const char *str, ssize_t *val);
-void align(char **strp);
+bool parse_line_column(const char *str, ssize_t *line, ssize_t *column);
+void align(char **str);
 void null_at(char **data, size_t index);
 void unsunder(char *str, size_t true_len);
 void sunder(char *str);
@@ -569,7 +585,7 @@ void remove_magicline(void);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
 #endif
-void get_totals(const filestruct *begin, const filestruct *end, int
+void get_totals(const filestruct *begin, const filestruct *end, size_t
 	*lines, size_t *size);
 
 /* Public functions in winio.c. */
@@ -625,8 +641,8 @@ void do_statusbar_backspace(void);
 void do_statusbar_delete(void);
 void do_statusbar_cut_text(void);
 #ifndef NANO_SMALL
-void do_statusbar_next_word(void);
-void do_statusbar_prev_word(void);
+bool do_statusbar_next_word(bool allow_punct);
+bool do_statusbar_prev_word(bool allow_punct);
 #endif
 void do_statusbar_verbatim_input(bool *got_enter);
 void do_statusbar_output(char *output, size_t output_len, bool
@@ -644,29 +660,26 @@ void check_statusblank(void);
 char *display_string(const char *buf, size_t start_col, size_t len, bool
 	dollars);
 void nanoget_repaint(const char *buf, const char *inputbuf, size_t x);
-int nanogetstr(bool allow_tabs, const char *buf, const char *def,
+int nanogetstr(bool allow_tabs, const char *buf, const char *curranswer,
 #ifndef NANO_SMALL
-		historyheadtype *history_list,
+	filestruct **history_list,
 #endif
-		const shortcut *s
+	const shortcut *s
 #ifndef DISABLE_TABCOMP
-		, bool *list
+	, bool *list
 #endif
-		);
-int statusq(bool allow_tabs, const shortcut *s, const char *def,
+	);
+int statusq(bool allow_tabs, const shortcut *s, const char *curranswer,
 #ifndef NANO_SMALL
-		historyheadtype *history_list,
+	filestruct **history_list,
 #endif
-		const char *msg, ...);
+	const char *msg, ...);
 void statusq_abort(void);
 void titlebar(const char *path);
 void set_modified(void);
 void statusbar(const char *msg, ...);
 void bottombars(const shortcut *s);
 void onekey(const char *keystroke, const char *desc, size_t len);
-#ifndef NDEBUG
-int check_linenumbers(const filestruct *fileptr);
-#endif
 size_t get_page_start(size_t column);
 void reset_cursor(void);
 void edit_add(const filestruct *fileptr, const char *converted, int
@@ -679,7 +692,7 @@ void edit_redraw(const filestruct *old_current, size_t old_pww);
 void edit_refresh(void);
 void edit_update(topmidnone location);
 int do_yesno(bool all, const char *msg);
-void total_update(void);
+void total_redraw(void);
 void total_refresh(void);
 void display_main_list(void);
 void do_cursorpos(bool constant);
@@ -688,7 +701,10 @@ void do_cursorpos_void(void);
 size_t help_line_len(const char *ptr);
 void do_help(void);
 #endif
-void do_replace_highlight(bool highlight_flag, const char *word);
+void do_replace_highlight(bool highlight, const char *word);
+#ifndef NDEBUG
+int check_linenumbers(const filestruct *fileptr);
+#endif
 #ifdef DEBUG
 void dump_buffer(const filestruct *inptr);
 void dump_buffer_reverse(void);
@@ -696,3 +712,5 @@ void dump_buffer_reverse(void);
 #ifdef NANO_EXTRA
 void do_credits(void);
 #endif
+
+#endif /* !PROTO_H */
