@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.49 2005/03/27 01:34:40 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.69 2005/06/28 06:25:34 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -32,29 +33,17 @@
 #include <ctype.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 #ifdef ENABLE_NANORC
 
 const static rcoption rcopts[] = {
-#ifndef NANO_SMALL
-    {"autoindent", AUTOINDENT},
-    {"backup", BACKUP_FILE},
-    {"backupdir", 0},
-#endif
 #ifndef DISABLE_JUSTIFY
     {"brackets", 0},
 #endif
-    {"const", CONSTUPDATE},
-#ifndef NANO_SMALL
-    {"cut", CUT_TO_END},
-#endif
+    {"const", CONST_UPDATE},
 #ifndef DISABLE_WRAPJUSTIFY
     {"fill", 0},
 #endif
-#ifndef NANO_SMALL
-    {"historylog", HISTORYLOG},
-#endif
 #ifndef DISABLE_MOUSE
     {"mouse", USE_MOUSE},
 #endif
@@ -62,9 +51,6 @@ const static rcoption rcopts[] = {
     {"multibuffer", MULTIBUFFER},
 #endif
     {"morespace", MORE_SPACE},
-#ifndef NANO_SMALL
-    {"noconvert", NO_CONVERT},
-#endif
     {"nofollow", NOFOLLOW_SYMLINKS},
     {"nohelp", NO_HELP},
 #ifndef DISABLE_WRAPPING
@@ -82,10 +68,6 @@ const static rcoption rcopts[] = {
 #ifdef HAVE_REGEX_H
     {"regexp", USE_REGEXP},
 #endif
-#ifndef NANO_SMALL
-    {"smarthome", SMART_HOME},
-    {"smooth", SMOOTHSCROLL},
-#endif
 #ifndef DISABLE_SPELLER
     {"speller", 0},
 #endif
@@ -94,13 +76,25 @@ const static rcoption rcopts[] = {
     {"tempfile", TEMP_FILE},
     {"view", VIEW_MODE},
 #ifndef NANO_SMALL
+    {"autoindent", AUTOINDENT},
+    {"backup", BACKUP_FILE},
+    {"backupdir", 0},
+    {"backwards", BACKWARDS_SEARCH},
+    {"casesensitive", CASE_SENSITIVE},
+    {"cut", CUT_TO_END},
+    {"historylog", HISTORYLOG},
+    {"noconvert", NO_CONVERT},
+    {"quickblank", QUICK_BLANK},
+    {"smarthome", SMART_HOME},
+    {"smooth", SMOOTH_SCROLL},
+    {"tabstospaces", TABS_TO_SPACES},
     {"whitespace", 0},
 #endif
     {NULL, 0}
 };
 
 static bool errors = FALSE;
-static int lineno = 0;
+static size_t lineno = 0;
 static char *nanorc = NULL;
 #ifdef ENABLE_COLOR
 static syntaxtype *endsyntax = NULL;
@@ -110,7 +104,7 @@ static colortype *endcolor = NULL;
 #endif
 
 /* We have an error in some part of the rcfile.  Put it on stderr and
- * make the user hit return to continue starting up nano. */
+ * make the user hit Return to continue starting up nano. */
 void rcfile_error(const char *msg, ...)
 {
     va_list ap;
@@ -118,7 +112,8 @@ void rcfile_error(const char *msg, ...)
     fprintf(stderr, "\n");
     if (lineno > 0) {
 	errors = TRUE;
-	fprintf(stderr, _("Error in %s on line %d: "), nanorc, lineno);
+	fprintf(stderr, _("Error in %s on line %lu: "), nanorc,
+		(unsigned long)lineno);
     }
 
     va_start(ap, msg);
@@ -132,7 +127,7 @@ void rcfile_error(const char *msg, ...)
  * the end of the line. */
 char *parse_next_word(char *ptr)
 {
-    while (!is_blank_char(*ptr) && *ptr != '\0')
+    while (!isblank(*ptr) && *ptr != '\0')
 	ptr++;
 
     if (*ptr == '\0')
@@ -141,7 +136,7 @@ char *parse_next_word(char *ptr)
     /* Null-terminate and advance ptr. */
     *ptr++ = '\0';
 
-    while (is_blank_char(*ptr))
+    while (isblank(*ptr))
 	ptr++;
 
     return ptr;
@@ -181,7 +176,7 @@ char *parse_argument(char *ptr)
 	ptr = last_quote + 1;
     }
     if (ptr != NULL)
-	while (is_blank_char(*ptr))
+	while (isblank(*ptr))
 	    ptr++;
     return ptr;
 }
@@ -191,12 +186,7 @@ int color_to_int(const char *colorname, bool *bright)
 {
     int mcolor = -1;
 
-    if (colorname == NULL) {
-	rcfile_error(N_("Missing color name"));
-	return -1;
-    }
-
-    assert(bright != NULL);
+    assert(colorname != NULL && bright != NULL);
 
     if (strncasecmp(colorname, "bright", 6) == 0) {
 	*bright = TRUE;
@@ -235,7 +225,7 @@ char *parse_next_regex(char *ptr)
 
     /* Continue until the end of the line, or a " followed by a space, a
      * blank character, or \0. */
-    while ((*ptr != '"' || (!is_blank_char(*(ptr + 1)) &&
+    while ((*ptr != '"' || (!isblank(*(ptr + 1)) &&
 	*(ptr + 1) != '\0')) && *ptr != '\0')
 	ptr++;
 
@@ -250,7 +240,7 @@ char *parse_next_regex(char *ptr)
     /* Null terminate and advance ptr. */
     *ptr++ = '\0';
 
-    while (is_blank_char(*ptr))
+    while (isblank(*ptr))
 	ptr++;
 
     return ptr;
@@ -352,11 +342,12 @@ void parse_syntax(char *ptr)
     }
 }
 
-/* Parse the color stuff into the colorstrings array. */
-void parse_colors(char *ptr)
+/* Parse the color stuff into the colorstrings array.  If icase is TRUE,
+ * treat the color stuff as case insensitive. */
+void parse_colors(char *ptr, bool icase)
 {
     int fg, bg;
-    bool bright = FALSE;
+    bool bright = FALSE, no_fgcolor = FALSE;
     char *fgstr;
 
     assert(ptr != NULL);
@@ -371,8 +362,15 @@ void parse_colors(char *ptr)
 
     if (strchr(fgstr, ',') != NULL) {
 	char *bgcolorname;
+
 	strtok(fgstr, ",");
 	bgcolorname = strtok(NULL, ",");
+	if (bgcolorname == NULL) {
+	    /* If we have a background color without a foreground color,
+	     * parse it properly. */
+	    bgcolorname = fgstr + 1;
+	    no_fgcolor = TRUE;
+	}
 	if (strncasecmp(bgcolorname, "bright", 6) == 0) {
 	    rcfile_error(
 		N_("Background color %s cannot be bright"),
@@ -383,11 +381,14 @@ void parse_colors(char *ptr)
     } else
 	bg = -1;
 
-    fg = color_to_int(fgstr, &bright);
+    if (!no_fgcolor) {
+	fg = color_to_int(fgstr, &bright);
 
-    /* Don't try to parse screwed-up foreground colors. */
-    if (fg == -1)
-	return;
+	/* Don't try to parse screwed-up foreground colors. */
+	if (fg == -1)
+	    return;
+    } else
+	fg = -1;
 
     if (syntaxes == NULL) {
 	rcfile_error(
@@ -395,9 +396,13 @@ void parse_colors(char *ptr)
 	return;
     }
 
-    /* Now for the fun part.  Start adding regexps to individual strings
-     * in the colorstrings array, woo! */
+    if (*ptr == '\0') {
+	rcfile_error(N_("Missing regex string"));
+	return;
+    }
 
+    /* Now for the fun part.  Start adding regexes to individual strings
+     * in the colorstrings array, woo! */
     while (ptr != NULL && *ptr != '\0') {
 	colortype *newcolor;
 	    /* The new color structure. */
@@ -426,7 +431,7 @@ void parse_colors(char *ptr)
 	if (ptr == NULL)
 	    break;
 
-	if (nregcomp(&newcolor->start, fgstr, 0)) {
+	if (nregcomp(&newcolor->start, fgstr, icase ? REG_ICASE : 0)) {
 	    free(newcolor);
 	    cancelled = TRUE;
 	} else {
@@ -476,7 +481,7 @@ void parse_colors(char *ptr)
 		continue;
 
 	    newcolor->end = (regex_t *)nmalloc(sizeof(regex_t));
-	    if (nregcomp(newcolor->end, fgstr, 0)) {
+	    if (nregcomp(newcolor->end, fgstr, icase ? REG_ICASE : 0)) {
 		free(newcolor->end);
 		newcolor->end = NULL;
 	    }
@@ -501,7 +506,7 @@ void parse_rcfile(FILE *rcstream)
 
 	lineno++;
 	ptr = buf;
-	while (is_blank_char(*ptr))
+	while (isblank(*ptr))
 	    ptr++;
 
 	/* If we have a blank line or a comment, skip to the next
@@ -522,7 +527,9 @@ void parse_rcfile(FILE *rcstream)
 	else if (strcasecmp(keyword, "syntax") == 0)
 	    parse_syntax(ptr);
 	else if (strcasecmp(keyword, "color") == 0)
-	    parse_colors(ptr);
+	    parse_colors(ptr, FALSE);
+	else if (strcasecmp(keyword, "icolor") == 0)
+	    parse_colors(ptr, TRUE);
 #endif /* ENABLE_COLOR */
 	else
 	    rcfile_error(N_("Command %s not understood"), keyword);
@@ -561,10 +568,20 @@ void parse_rcfile(FILE *rcstream)
 			if (*option == '"')
 			    option++;
 			ptr = parse_argument(ptr);
-			option = make_mbstring(option);
+
+			option = mallocstrcpy(NULL, option);
 #ifdef DEBUG
 			fprintf(stderr, "option = \"%s\"\n", option);
 #endif
+
+			/* Make sure option is a valid multibyte
+			 * string. */
+			if (!is_valid_mbstring(option)) {
+			    rcfile_error(
+				N_("Option is not a valid multibyte string"));
+			    break;
+			}
+
 #ifndef DISABLE_OPERATINGDIR
 			if (strcasecmp(rcopts[i].name, "operatingdir") == 0)
 			    operating_dir = option;
@@ -584,7 +601,8 @@ void parse_rcfile(FILE *rcstream)
 #ifndef NANO_SMALL
 			if (strcasecmp(rcopts[i].name, "whitespace") == 0) {
 			    whitespace = option;
-			    if (mbstrlen(whitespace) != 2 || strlenpt(whitespace) != 2) {
+			    if (mbstrlen(whitespace) != 2 ||
+				strlenpt(whitespace) != 2) {
 				rcfile_error(
 					N_("Two single-column characters required"));
 				free(whitespace);
@@ -603,20 +621,18 @@ void parse_rcfile(FILE *rcstream)
 #ifndef DISABLE_JUSTIFY
 			if (strcasecmp(rcopts[i].name, "punct") == 0) {
 			    punct = option;
-			    if (strchr(punct, '\t') != NULL ||
-				strchr(punct, ' ') != NULL) {
+			    if (has_blank_mbchars(punct)) {
 				rcfile_error(
-					N_("Non-tab and non-space characters required"));
+					N_("Non-blank characters required"));
 				free(punct);
 				punct = NULL;
 			    }
 			} else if (strcasecmp(rcopts[i].name,
 				"brackets") == 0) {
 			    brackets = option;
-			    if (strchr(brackets, '\t') != NULL ||
-				strchr(brackets, ' ') != NULL) {
+			    if (has_blank_mbchars(brackets)) {
 				rcfile_error(
-					N_("Non-tab and non-space characters required"));
+					N_("Non-blank characters required"));
 				free(brackets);
 				brackets = NULL;
 			    }
