@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.93 2005/03/14 19:17:27 dolorous Exp $ */
+/* $Id: search.c,v 1.140 2005/06/29 00:17:18 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -31,10 +32,13 @@
 #include <errno.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 static bool search_last_line = FALSE;
 	/* Have we gone past the last line while searching? */
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+static bool history_changed = FALSE;
+	/* Have any of the history lists changed? */
+#endif
 #ifdef HAVE_REGEX_H
 static bool regexp_compiled = FALSE;
 	/* Have we compiled any regular expressions? */
@@ -145,10 +149,6 @@ int search_init(bool replacing, bool use_answer)
 
     search_init_globals();
 
-#ifndef NANO_SMALL
-    search_history.current = (historytype *)&search_history.next;
-#endif
-
     if (last_search[0] != '\0') {
 	char *disp = display_string(last_search, 0, COLS / 3, FALSE);
 
@@ -186,7 +186,7 @@ int search_init(bool replacing, bool use_answer)
 #ifndef NANO_SMALL
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
-	ISSET(REVERSE_SEARCH) ? _(" [Backwards]") :
+	ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") :
 #endif
 		"",
 
@@ -212,9 +212,6 @@ int search_init(bool replacing, bool use_answer)
     if (i == -1 || (i < 0 && last_search[0] == '\0') ||
 	    (!replacing && i == 0 && answer[0] == '\0')) {
 	statusbar(_("Cancelled"));
-#ifndef NANO_SMALL
-	search_history.current = search_history.next;
-#endif
 	return -1;
     } else {
 	switch (i) {
@@ -238,30 +235,29 @@ int search_init(bool replacing, bool use_answer)
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 	    case TOGGLE_BACKWARDS_KEY:
-		TOGGLE(REVERSE_SEARCH);
+		TOGGLE(BACKWARDS_SEARCH);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
+#endif
 #ifdef HAVE_REGEX_H
-	    case TOGGLE_REGEXP_KEY:
+	    case NANO_REGEXP_KEY:
 		TOGGLE(USE_REGEXP);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 #endif
-#endif /* !NANO_SMALL */
 	    case NANO_TOOTHERSEARCH_KEY:
 		backupstring = mallocstrcpy(backupstring, answer);
 		return -2;	/* Call the opposite search function. */
 	    case NANO_TOGOTOLINE_KEY:
-#ifndef NANO_SMALL
-		search_history.current = search_history.next;
-#endif
-		do_gotoline(-1, FALSE);	/* Put answer up on the
-					 * statusbar and fall
-					 * through. */
+		do_gotolinecolumn(current->lineno, placewewant + 1,
+			TRUE, TRUE, FALSE);
+				/* Put answer up on the statusbar and
+				 * fall through. */
 	    default:
 		return -1;
 	}
     }
+
     return 0;
 }
 
@@ -277,11 +273,11 @@ bool is_whole_word(size_t pos, const char *buf, const char *word)
     parse_mbchar(buf + word_end, r, NULL, NULL);
 
     /* If we're at the beginning of the line or the character before the
-     * word isn't an alphanumeric character, and if we're at the end of
-     * the line or the character after the word isn't an alphanumeric
-     * character, we have a whole word. */
-    retval = (pos == 0 || !is_alnum_mbchar(p)) &&
-	(word_end == strlen(buf) || !is_alnum_mbchar(r));
+     * word isn't a non-punctuation "word" character, and if we're at
+     * the end of the line or the character after the word isn't a
+     * non-punctuation "word" character, we have a whole word. */
+    retval = (pos == 0 || !is_word_mbchar(p, FALSE)) &&
+	(word_end == strlen(buf) || !is_word_mbchar(r, FALSE));
 
     free(p);
     free(r);
@@ -306,7 +302,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 	/* The length of the match we found. */
     size_t current_x_find = 0;
 	/* The location of the match we found. */
-    int current_y_find = current_y;
+    ssize_t current_y_find = current_y;
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because strstrwrapper()
@@ -315,7 +311,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
      * previous or next line. */
     rev_start =
 #ifndef NANO_SMALL
-	ISSET(REVERSE_SEARCH) ? fileptr->data + (current_x - 1) :
+	ISSET(BACKWARDS_SEARCH) ? fileptr->data + (current_x - 1) :
 #endif
 	fileptr->data + (current_x + 1);
 
@@ -364,7 +360,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 	}
 
 #ifndef NANO_SMALL
-	if (ISSET(REVERSE_SEARCH)) {
+	if (ISSET(BACKWARDS_SEARCH)) {
 	    fileptr = fileptr->prev;
 	    current_y_find--;
 	} else {
@@ -381,7 +377,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 		return FALSE;
 
 #ifndef NANO_SMALL
-	    if (ISSET(REVERSE_SEARCH)) {
+	    if (ISSET(BACKWARDS_SEARCH)) {
 		fileptr = filebot;
 		current_y_find = editwinrows - 1;
 	    } else {
@@ -402,7 +398,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 
 	rev_start = fileptr->data;
 #ifndef NANO_SMALL
-	if (ISSET(REVERSE_SEARCH))
+	if (ISSET(BACKWARDS_SEARCH))
 	    rev_start += strlen(fileptr->data);
 #endif
     }
@@ -413,13 +409,12 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
     /* Ensure we haven't wrapped around again! */
     if (search_last_line &&
 #ifndef NANO_SMALL
-	((!ISSET(REVERSE_SEARCH) && current_x_find > beginx) ||
-	(ISSET(REVERSE_SEARCH) && current_x_find < beginx))
+	((!ISSET(BACKWARDS_SEARCH) && current_x_find > beginx) ||
+	(ISSET(BACKWARDS_SEARCH) && current_x_find < beginx))
 #else
 	current_x_find > beginx
 #endif
 	) {
-
 	if (can_display_wrap)
 	    not_found_msg(needle);
 	return FALSE;
@@ -461,7 +456,7 @@ void do_search(void)
 	search_abort();
     else if (i == -2)	/* Replace. */
 	do_replace();
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) || defined(HAVE_REGEX_H)
     else if (i == 1)	/* Case Sensitive, Backwards, or Regexp search
 			 * toggle. */
 	do_search();
@@ -554,7 +549,8 @@ void do_research(void)
 		last_search)) {
 		didfind = findnextstr(TRUE, FALSE, TRUE, current,
 			current_x, answer, NULL);
-		if (fileptr == current && fileptr_x == current_x && !didfind)
+		if (fileptr == current && fileptr_x == current_x &&
+			!didfind)
 		    statusbar(_("This is the only occurrence"));
 	    } else {
 #endif
@@ -582,15 +578,17 @@ void replace_abort(void)
 }
 
 #ifdef HAVE_REGEX_H
-int replace_regexp(char *string, bool create_flag)
+int replace_regexp(char *string, bool create)
 {
-    /* Split personality here - if create_flag is FALSE, just calculate
-     * the size of the replacement line (necessary because of
+    /* We have a split personality here.  If create is FALSE, just
+     * calculate the size of the replacement line (necessary because of
      * subexpressions \1 to \9 in the replaced text). */
 
     const char *c = last_replace;
-    int search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
-    int new_size = strlen(current->data) + 1 - search_match_count;
+    size_t search_match_count =
+	regmatches[0].rm_eo - regmatches[0].rm_so;
+    size_t new_line_size =
+	strlen(current->data) + 1 - search_match_count;
 
     /* Iterate through the replacement text to handle subexpression
      * replacement using \1, \2, \3, etc. */
@@ -599,22 +597,22 @@ int replace_regexp(char *string, bool create_flag)
 
 	if (*c != '\\' || num < 1 || num > 9 ||
 		num > search_regexp.re_nsub) {
-	    if (create_flag)
+	    if (create)
 		*string++ = *c;
 	    c++;
-	    new_size++;
+	    new_line_size++;
 	} else {
-	    int i = regmatches[num].rm_eo - regmatches[num].rm_so;
+	    size_t i = regmatches[num].rm_eo - regmatches[num].rm_so;
 
 	    /* Skip over the replacement expression. */
 	    c += 2;
 
 	    /* But add the length of the subexpression to new_size. */
-	    new_size += i;
+	    new_line_size += i;
 
-	    /* And if create_flag is TRUE, append the result of the
+	    /* And if create is TRUE, append the result of the
 	     * subexpression match to the new line. */
-	    if (create_flag) {
+	    if (create) {
 		strncpy(string, current->data + current_x +
 			regmatches[num].rm_so, i);
 		string += i;
@@ -622,24 +620,23 @@ int replace_regexp(char *string, bool create_flag)
 	}
     }
 
-    if (create_flag)
+    if (create)
 	*string = '\0';
 
-    return new_size;
+    return new_line_size;
 }
 #endif
 
 char *replace_line(const char *needle)
 {
     char *copy;
-    int new_line_size;
-    int search_match_count;
+    size_t new_line_size, search_match_count;
 
     /* Calculate the size of the new line. */
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
 	search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
-	new_line_size = replace_regexp(NULL, 0);
+	new_line_size = replace_regexp(NULL, FALSE);
     } else {
 #endif
 	search_match_count = strlen(needle);
@@ -665,6 +662,7 @@ char *replace_line(const char *needle)
 
     /* The tail of the original line. */
     assert(current_x + search_match_count <= strlen(current->data));
+
     strcat(copy, current->data + current_x + search_match_count);
 
     return copy;
@@ -684,7 +682,6 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 {
     ssize_t numreplaced = -1;
     size_t match_len;
-    size_t pww_save = placewewant;
     bool replaceall = FALSE;
 #ifdef HAVE_REGEX_H
     /* The starting-line match and bol/eol regex flags. */
@@ -726,7 +723,6 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	FALSE
 #endif
 	, real_current, *real_current_x, needle, &match_len)) {
-
 	int i = 0;
 
 #ifdef HAVE_REGEX_H
@@ -746,31 +742,29 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	}
 #endif
 
-	if (!replaceall) {
-	    edit_redraw(real_current, pww_save);
-	    pww_save = placewewant;
-	}
+	if (!replaceall)
+	    edit_refresh();
 
-	/* Record for the return value that we found the search
-	 * string. */
+	/* Indicate that we found the search string. */
 	if (numreplaced == -1)
 	    numreplaced = 0;
 
 	if (!replaceall) {
-	    char *exp_word;
 	    size_t xpt = xplustabs();
-
-	    exp_word = display_string(current->data, xpt,
-		strnlenpt(current->data, match_len + current_x) - xpt,
+	    char *exp_word = display_string(current->data, xpt,
+		strnlenpt(current->data, current_x + match_len) - xpt,
 		FALSE);
 
 	    curs_set(0);
+
 	    do_replace_highlight(TRUE, exp_word);
 
 	    i = do_yesno(TRUE, _("Replace this instance?"));
 
 	    do_replace_highlight(FALSE, exp_word);
+
 	    free(exp_word);
+
 	    curs_set(1);
 
 	    if (i == -1) {	/* We canceled the replace. */
@@ -833,7 +827,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	     * text.  Note that current_x might be set to (size_t)-1
 	     * here. */
 #ifndef NANO_SMALL
-	    if (!ISSET(REVERSE_SEARCH))
+	    if (!ISSET(BACKWARDS_SEARCH))
 #endif
 		current_x += match_len + length_change - 1;
 
@@ -913,11 +907,6 @@ void do_replace(void)
 	last_search = mallocstrcpy(last_search, answer);
     }
 
-#ifndef NANO_SMALL
-    replace_history.current = (historytype *)&replace_history.next;
-    last_replace = mallocstrcpy(last_replace, "");
-#endif
-
     i = statusq(FALSE, replace_list_2, last_replace,
 #ifndef NANO_SMALL
 	&replace_history,
@@ -962,21 +951,26 @@ void do_replace(void)
     edit_refresh();
 
     if (numreplaced >= 0)
-	statusbar(P_("Replaced %ld occurrence", "Replaced %ld occurrences",
-		(long)numreplaced), (long)numreplaced);
+	statusbar(P_("Replaced %lu occurrence",
+		"Replaced %lu occurrences", (unsigned long)numreplaced),
+		(unsigned long)numreplaced);
 
     replace_abort();
 }
 
-void do_gotoline(int line, bool save_pos)
+/* Go to the specified line and column, or ask for them if interactive
+ * is TRUE.  Save the x-coordinate and y-coordinate if save_pos is TRUE.
+ * Note that both the line and column numbers should be one-based. */
+void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
+	bool interactive, bool save_pos)
 {
-    if (line <= 0) {		/* Ask for it. */
+    if (interactive) {		/* Ask for it. */
 	char *ans = mallocstrcpy(NULL, answer);
-	int i = statusq(FALSE, gotoline_list, line < 0 ? ans : "",
+	int i = statusq(FALSE, gotoline_list, use_answer ? ans : "",
 #ifndef NANO_SMALL
 		NULL,
 #endif
-		_("Enter line number"));
+		_("Enter line number, column number"));
 
 	free(ans);
 
@@ -996,52 +990,50 @@ void do_gotoline(int line, bool save_pos)
 	}
 
 	/* Do a bounds check.  Display a warning on an out-of-bounds
-	 * line number only if we hit Enter at the statusbar prompt. */
-	if (!parse_num(answer, &line) || line < 0) {
+	 * line or column number only if we hit Enter at the statusbar
+	 * prompt. */
+	if (!parse_line_column(answer, &line, &column) || line < 1 ||
+		column < 1) {
 	    if (i == 0)
 		statusbar(_("Come on, be reasonable"));
 	    display_main_list();
 	    return;
 	}
-    }
-
-    if (current->lineno > line) {
-	for (; current->prev != NULL && current->lineno > line;
-		current = current->prev)
-	    ;
     } else {
-	for (; current->next != NULL && current->lineno < line;
-		current = current->next)
-	    ;
+	if (line < 1)
+	    line = current->lineno;
+
+	if (column < 1)
+	    column = placewewant + 1;
     }
 
-    current_x = 0;
+    for (current = fileage; current->next != NULL && line > 1; line--)
+	current = current->next;
+
+    current_x = actual_x(current->data, column - 1);
+    placewewant = column - 1;
 
     /* If save_pos is TRUE, don't change the cursor position when
      * updating the edit window. */
     edit_update(save_pos ? NONE : CENTER);
 
-    placewewant = 0;
     display_main_list();
 }
 
-void do_gotoline_void(void)
+void do_gotolinecolumn_void(void)
 {
-    do_gotoline(0, FALSE);
+    do_gotolinecolumn(current->lineno, placewewant + 1, FALSE, TRUE,
+	FALSE);
 }
 
 #if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
-void do_gotopos(int line, size_t pos_x, int pos_y, size_t pos_pww)
+void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
+	pos_pww)
 {
-    /* Since do_gotoline() resets the x-coordinate but not the
+    /* Since do_gotolinecolumn() resets the x-coordinate but not the
      * y-coordinate, set the coordinates up this way. */
     current_y = pos_y;
-    do_gotoline(line, TRUE);
-
-    /* Make sure that the x-coordinate is sane here. */
-    current_x = strlen(current->data);
-    if (pos_x < current_x)
-	current_x = pos_x;
+    do_gotolinecolumn(line, pos_x + 1, FALSE, FALSE, TRUE);
 
     /* Set the rest of the coordinates up. */
     placewewant = pos_pww;
@@ -1049,52 +1041,53 @@ void do_gotopos(int line, size_t pos_x, int pos_y, size_t pos_pww)
 }
 #endif
 
-#if !defined(NANO_SMALL) && defined(HAVE_REGEX_H)
+#ifndef NANO_SMALL
+#ifdef HAVE_REGEX_H
 void do_find_bracket(void)
 {
-    char ch_under_cursor, wanted_ch;
-    const char *pos, *brackets = "([{<>}])";
-    char regexp_pat[] = "[  ]";
+    const char *pos, *bracket_pat = "([{<>}])";
+    char cursor_ch, wanted_ch, regexp_pat[] = "[  ]";
     size_t current_x_save, pww_save;
     int count = 1;
-    unsigned long flags_save;
+    bool regexp_set = ISSET(USE_REGEXP);
+    bool backwards_search_set = ISSET(BACKWARDS_SEARCH);
     filestruct *current_save;
 
-    ch_under_cursor = current->data[current_x];
+    cursor_ch = current->data[current_x];
+    pos = strchr(bracket_pat, cursor_ch);
 
-    pos = strchr(brackets, ch_under_cursor);
-    if (ch_under_cursor == '\0' || pos == NULL) {
+    if (cursor_ch == '\0' || pos == NULL) {
 	statusbar(_("Not a bracket"));
 	return;
     }
 
-    assert(strlen(brackets) % 2 == 0);
+    assert(strlen(bracket_pat) % 2 == 0);
 
-    wanted_ch = brackets[(strlen(brackets) - 1) - (pos - brackets)];
+    wanted_ch =
+	bracket_pat[(strlen(bracket_pat) - 1) - (pos - bracket_pat)];
 
     current_save = current;
     current_x_save = current_x;
     pww_save = placewewant;
-    flags_save = flags;
     SET(USE_REGEXP);
 
     /* Apparent near redundancy with regexp_pat[] here is needed.
      * "[][]" works, "[[]]" doesn't. */
-    if (pos < brackets + (strlen(brackets) / 2)) {
+    if (pos < bracket_pat + (strlen(bracket_pat) / 2)) {
 	/* On a left bracket. */
 	regexp_pat[1] = wanted_ch;
-	regexp_pat[2] = ch_under_cursor;
-	UNSET(REVERSE_SEARCH);
+	regexp_pat[2] = cursor_ch;
+	UNSET(BACKWARDS_SEARCH);
     } else {
 	/* On a right bracket. */
-	regexp_pat[1] = ch_under_cursor;
+	regexp_pat[1] = cursor_ch;
 	regexp_pat[2] = wanted_ch;
-	SET(REVERSE_SEARCH);
+	SET(BACKWARDS_SEARCH);
     }
 
     regexp_init(regexp_pat);
 
-    /* We constructed regexp_pat to be a valid expression. */
+    /* We constructed regexp_pat to be a valid regular expression. */
     assert(regexp_compiled);
 
     findnextstr_wrap_reset();
@@ -1102,7 +1095,7 @@ void do_find_bracket(void)
 	if (findnextstr(FALSE, FALSE, FALSE, current, current_x,
 		regexp_pat, NULL)) {
 	    /* Found identical bracket. */
-	    if (current->data[current_x] == ch_under_cursor)
+	    if (current->data[current_x] == cursor_ch)
 		count++;
 	    /* Found complementary bracket. */
 	    else if (--count == 0) {
@@ -1121,133 +1114,201 @@ void do_find_bracket(void)
     }
 
     regexp_cleanup();
-    flags = flags_save;
+
+    /* Restore search direction. */
+    if (backwards_search_set)
+	SET(BACKWARDS_SEARCH);
+    else
+	UNSET(BACKWARDS_SEARCH);
+
+    /* Restore regular expression usage setting. */
+    if (!regexp_set)
+	UNSET(USE_REGEXP);
+}
+#endif /* HAVE_REGEX_H */
+
+#ifdef ENABLE_NANORC
+/* Indicate whether any of the history lists have changed. */
+bool history_has_changed(void)
+{
+    return history_changed;
 }
 #endif
 
-#ifndef NANO_SMALL
-/*
- * search and replace history list support functions
- */
-
-/* initialize search and replace history lists */
+/* Initialize the search and replace history lists. */
 void history_init(void)
 {
-    search_history.next = (historytype *)&search_history.prev;
-    search_history.prev = NULL;
-    search_history.tail = (historytype *)&search_history.next;
-    search_history.current = search_history.next;
-    search_history.count = 0;
-    search_history.len = 0;
+    search_history = make_new_node(NULL);
+    search_history->data = mallocstrcpy(NULL, "");
+    searchage = search_history;
+    searchbot = search_history;
 
-    replace_history.next = (historytype *)&replace_history.prev;
-    replace_history.prev = NULL;
-    replace_history.tail = (historytype *)&replace_history.next;
-    replace_history.current = replace_history.next;
-    replace_history.count = 0;
-    replace_history.len = 0;
+    replace_history = make_new_node(NULL);
+    replace_history->data = mallocstrcpy(NULL, "");
+    replaceage = replace_history;
+    replacebot = replace_history;
 }
 
-/* find first node containing string *s in history list *h */
-historytype *find_node(historytype *h, const char *s)
+/* Return the first node containing the first len characters of the
+ * string s in the history list, starting at h_start and ending at
+ * h_end, or NULL if there isn't one. */
+filestruct *find_history(filestruct *h_start, filestruct *h_end, const
+	char *s, size_t len)
 {
-    for (; h->next != NULL; h = h->next)
-	if (strcmp(s, h->data) == 0)
-	    return h;
+    filestruct *p;
+
+    for (p = h_start; p != h_end->next && p != NULL; p = p->next) {
+	if (strncmp(s, p->data, len) == 0)
+	    return p;
+    }
+
     return NULL;
 }
 
-/* remove node *r */
-void remove_node(historytype *r)
+/* Update a history list.  h should be the current position in the
+ * list. */
+void update_history(filestruct **h, const char *s)
 {
-    r->prev->next = r->next;
-    r->next->prev = r->prev;
-    free(r->data);
-    free(r);
-}
+    filestruct **hage = NULL, **hbot = NULL, *p;
 
-/* add a node after node *h */
-void insert_node(historytype *h, const char *s)
-{
-    historytype *a;
+    assert(h != NULL && s != NULL);
 
-    a = (historytype *)nmalloc(sizeof(historytype));
-    a->next = h->next;
-    a->prev = h;
-    h->next->prev = a;
-    h->next = a;
-    a->data = mallocstrcpy(NULL, s);
-}
-
-/* update history list */
-void update_history(historyheadtype *h, const char *s)
-{
-    historytype *p;
-
-    if ((p = find_node(h->next, s)) != NULL) {
-	if (p == h->next)		/* catch delete and re-insert of
-					   same string in 1st node */
-	    goto up_hs;
-	remove_node(p);			/* delete identical older string */
-	h->count--;
+    if (*h == search_history) {
+	hage = &searchage;
+	hbot = &searchbot;
+    } else if (*h == replace_history) {
+	hage = &replaceage;
+	hbot = &replacebot;
     }
-    if (h->count == MAX_SEARCH_HISTORY) {	/* list 'full', delete oldest */
-	remove_node(h->tail);
-	h->count--;
+
+    assert(hage != NULL && hbot != NULL);
+
+    /* If this string is already in the history, delete it. */
+    p = find_history(*hage, *hbot, s, (size_t)-1);
+
+    if (p != NULL) {
+	filestruct *foo, *bar;
+
+	/* If the string is at the beginning, move the beginning down to
+	 * the next string. */
+	if (p == *hage)
+	    *hage = (*hage)->next;
+
+	/* Delete the string. */
+	foo = p;
+	bar = p->next;
+	unlink_node(foo);
+	delete_node(foo);
+	renumber(bar);
     }
-    insert_node((historytype *)h, s);
-    h->count++;
-    SET(HISTORY_CHANGED);
-  up_hs:
-    h->current = h->next;
-}
 
-/* return a pointer to either the next older history or NULL if no more */
-char *get_history_older(historyheadtype *h)
-{
-    if (h->current->next != NULL) {	/* any older entries? */
-	h->current = h->current->next;	/* yes */
-	return h->current->data;	/* return it */
+    /* If the history is full, delete the beginning entry to make room
+     * for the new entry at the end. */
+    if ((*hbot)->lineno == MAX_SEARCH_HISTORY + 1) {
+	filestruct *foo = *hage;
+
+	*hage = (*hage)->next;
+	unlink_node(foo);
+	delete_node(foo);
+	renumber(*hage);
     }
-    return NULL;			/* end of list */
-}
 
-char *get_history_newer(historyheadtype *h)
-{
-    if (h->current->prev != NULL) {
-	h->current = h->current->prev;
-	if (h->current->prev != NULL)
-	    return h->current->data;
-    }
-    return NULL;
-}
+    /* Add the new entry to the end. */
+    (*hbot)->data = mallocstrcpy(NULL, s);
+    splice_node(*hbot, make_new_node(*hbot), (*hbot)->next);
+    *hbot = (*hbot)->next;
+    (*hbot)->data = mallocstrcpy(NULL, "");
 
-/* get a completion */
-char *get_history_completion(historyheadtype *h, char *s)
-{
-    historytype *p;
-
-    for (p = h->current->next; p->next != NULL; p = p->next) {
-	if (strncmp(s, p->data, h->len) == 0 && strlen(p->data) != h->len) {
-	    h->current = p;
-	    return p->data;
-	}
-    }
-    h->current = (historytype*)h;
-    null_at(&s, h->len);
-    return s;
-}
-
-#ifdef DEBUG
-/* free a history list */
-void free_history(historyheadtype *h)
-{
-    historytype *p;
-
-    for (p = h->next; p->next != NULL; p = p->next)
-	remove_node(p);
-}
+#ifdef ENABLE_NANORC
+    /* Indicate that the history's been changed. */
+    history_changed = TRUE;
 #endif
 
-/* end of history support functions */
+    /* Set the current position in the list to the bottom. */
+    *h = *hbot;
+}
+
+/* Move h to the string in the history list just before it, and return
+ * that string.  If there isn't one, don't move h and return NULL. */
+char *get_history_older(filestruct **h)
+{
+    assert(h != NULL);
+
+    if ((*h)->prev == NULL)
+	return NULL;
+
+    *h = (*h)->prev;
+
+    return (*h)->data;
+}
+
+/* Move h to the string in the history list just after it, and return
+ * that string.  If there isn't one, don't move h and return NULL. */
+char *get_history_newer(filestruct **h)
+{
+    assert(h != NULL);
+
+    if ((*h)->next == NULL)
+	return NULL;
+
+    *h = (*h)->next;
+
+    return (*h)->data;
+}
+
+#ifndef DISABLE_TABCOMP
+/* Move h to the next string that's a tab completion of the string s,
+ * looking at only the first len characters of s, and return that
+ * string.  If there isn't one, or if len is 0, don't move h and return
+ * s. */
+char *get_history_completion(filestruct **h, char *s, size_t len)
+{
+    assert(s != NULL);
+
+    if (len > 0) {
+	filestruct *hage = NULL, *hbot = NULL, *p;
+
+	assert(h != NULL);
+
+	if (*h == search_history) {
+	    hage = searchage;
+	    hbot = searchbot;
+	} else if (*h == replace_history) {
+	    hage = replaceage;
+	    hbot = replacebot;
+	}
+
+	assert(hage != NULL && hbot != NULL);
+
+	/* Search the history list from the current position to the
+	 * bottom for a match of len characters.  Skip over an exact
+	 * match. */
+	p = find_history((*h)->next, hbot, s, len);
+
+	while (p != NULL && strcmp(p->data, s) == 0)
+	    p = find_history(p->next, hbot, s, len);
+
+	if (p != NULL) {
+	    *h = p;
+	    return (*h)->data;
+	}
+
+	/* Search the history list from the top to the current position
+	 * for a match of len characters.  Skip over an exact match. */
+	p = find_history(hage, *h, s, len);
+
+	while (p != NULL && strcmp(p->data, s) == 0)
+	    p = find_history(p->next, *h, s, len);
+
+	if (p != NULL) {
+	    *h = p;
+	    return (*h)->data;
+	}
+    }
+
+    /* If we're here, we didn't find a match, we didn't find an inexact
+     * match, or len is 0.  Return s. */
+    return s;
+}
+#endif
 #endif /* !NANO_SMALL */
