@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.69 2005/04/08 20:06:08 dolorous Exp $ */
+/* $Id: utils.c,v 1.87 2005/06/28 06:25:34 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -8,14 +8,15 @@
  *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful,      *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *   GNU General Public License for more details.                         *
+ *   This program is distributed in the hope that it will be useful, but  *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
+ *   General Public License for more details.                             *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
  *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
+ *   02110-1301, USA.                                                     *
  *                                                                        *
  **************************************************************************/
 
@@ -32,7 +33,6 @@
 #include <errno.h>
 #include <assert.h>
 #include "proto.h"
-#include "nano.h"
 
 #ifdef HAVE_REGEX_H
 #ifdef BROKEN_REGEXEC
@@ -55,13 +55,10 @@ int regexp_bol_or_eol(const regex_t *preg, const char *string)
 }
 #endif /* HAVE_REGEX_H */
 
-int num_of_digits(int n)
+int digits(size_t n)
 {
     int i = 1;
 
-    if (n < 0)
-	n = -n;
-
     while (n > 10) {
 	n /= 10;
 	i++;
@@ -108,13 +105,46 @@ bool parse_num(const char *str, ssize_t *val)
     return TRUE;
 }
 
-/* Fix the memory allocation for a string. */
-void align(char **strp)
+/* Read an int and a ssize_t, separated by a comma, from str, and store
+ * them in *line and *column (if they're not both NULL).  On error, we
+ * return FALSE.  Otherwise, we return TRUE. */
+bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
 {
-    assert(strp != NULL);
+    bool retval = TRUE;
+    const char *comma;
 
-    if (*strp != NULL)
-	*strp = charealloc(*strp, strlen(*strp) + 1);
+    assert(str != NULL);
+
+    comma = strchr(str, ',');
+
+    if (comma != NULL && column != NULL) {
+	if (!parse_num(str + (comma - str + 1), column))
+	    retval = FALSE;
+    }
+
+    if (line != NULL) {
+	if (comma != NULL) {
+	    char *str_line = mallocstrncpy(NULL, str, comma - str + 1);
+	    str_line[comma - str] = '\0';
+
+	    if (str_line[0] != '\0' && !parse_num(str_line, line))
+		retval = FALSE;
+
+	    free(str_line);
+	} else if (!parse_num(str, line))
+	    retval = FALSE;
+    }
+
+    return retval;
+}
+
+/* Fix the memory allocation for a string. */
+void align(char **str)
+{
+    assert(str != NULL);
+
+    if (*str != NULL)
+	*str = charealloc(*str, strlen(*str) + 1);
 }
 
 /* Null a string at a certain index and align it. */
@@ -174,15 +204,15 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 
     /* Allocate the line the first time. */
     if (*lineptr == NULL) {
-	*lineptr = charalloc(128);
-	*n = 128;
+	*lineptr = charalloc(MAX_BUF_SIZE);
+	*n = MAX_BUF_SIZE;
     }
 
     while ((c = getc(stream)) != EOF) {
 	/* Check if more memory is needed. */
 	if (indx >= *n) {
-	    *lineptr = charealloc(*lineptr, *n + 128);
-	    *n += 128;
+	    *lineptr = charealloc(*lineptr, *n + MAX_BUF_SIZE);
+	    *n += MAX_BUF_SIZE;
 	}
 
 	/* Push the result in the line. */
@@ -195,8 +225,8 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 
     /* Make room for the null character. */
     if (indx >= *n) {
-	*lineptr = charealloc(*lineptr, *n + 128);
-	*n += 128;
+	*lineptr = charealloc(*lineptr, *n + MAX_BUF_SIZE);
+	*n += MAX_BUF_SIZE;
     }
 
     /* Null terminate the buffer. */
@@ -229,16 +259,16 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
 #ifndef NANO_SMALL
-	if (ISSET(REVERSE_SEARCH)) {
-	    if (regexec(&search_regexp, haystack, 1, regmatches, 0) == 0
-		&& haystack + regmatches[0].rm_so <= start) {
+	if (ISSET(BACKWARDS_SEARCH)) {
+	    if (regexec(&search_regexp, haystack, 1, regmatches,
+		0) == 0 && haystack + regmatches[0].rm_so <= start) {
 		const char *retval = haystack + regmatches[0].rm_so;
 
 		/* Search forward until there are no more matches. */
-		while (regexec(&search_regexp, retval + 1, 1, regmatches,
-			REG_NOTBOL) == 0 && retval + 1 +
-			regmatches[0].rm_so <= start)
-		    retval += 1 + regmatches[0].rm_so;
+		while (regexec(&search_regexp, retval + 1, 1,
+			regmatches, REG_NOTBOL) == 0 &&
+			retval + regmatches[0].rm_so + 1 <= start)
+		    retval += regmatches[0].rm_so + 1;
 		/* Finally, put the subexpression matches in global
 		 * variable regmatches.  The REG_NOTBOL flag doesn't
 		 * matter now. */
@@ -260,7 +290,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 #if !defined(NANO_SMALL) || !defined(DISABLE_SPELLER)
     if (ISSET(CASE_SENSITIVE)) {
 #ifndef NANO_SMALL
-	if (ISSET(REVERSE_SEARCH))
+	if (ISSET(BACKWARDS_SEARCH))
 	    return revstrstr(haystack, needle, start);
 	else
 #endif
@@ -268,7 +298,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
     }
 #endif /* !DISABLE_SPELLER || !NANO_SMALL */
 #ifndef NANO_SMALL
-    else if (ISSET(REVERSE_SEARCH))
+    else if (ISSET(BACKWARDS_SEARCH))
 	return mbrevstrcasestr(haystack, needle, start);
 #endif
     return mbstrcasestr(start, needle);
@@ -329,7 +359,8 @@ char *mallocstrncpy(char *dest, const char *src, size_t n)
  * "dest = mallocstrcpy(dest, src);". */
 char *mallocstrcpy(char *dest, const char *src)
 {
-    return mallocstrncpy(dest, src, src == NULL ? 1 : strlen(src) + 1);
+    return mallocstrncpy(dest, src, (src == NULL) ? 1 :
+	strlen(src) + 1);
 }
 
 /* Free the malloc()ed string at dest and return the malloc()ed string
@@ -399,7 +430,7 @@ void mark_order(const filestruct **top, size_t *top_x, const filestruct
 
 /* Calculate the number of lines and the number of characters between
  * begin and end, and return them in lines and size, respectively. */
-void get_totals(const filestruct *begin, const filestruct *end, int
+void get_totals(const filestruct *begin, const filestruct *end, size_t
 	*lines, size_t *size)
 {
     const filestruct *f;
